"""This module contains an autogenerated copy of the external SSSOM context."""

# This is autogenerated and super hacky.
sssom_external_context = """
{
    "@context": {
        "3dmet": "http://www.3dmet.dna.affrc.go.jp/cgi/show_data.php?acc=",
        "4dn.biosource": "https://data.4dnucleome.org/biosources/",
        "4dn.replicate": "https://data.4dnucleome.org/experiment-set-replicates/",
        "AAO": "http://purl.obolibrary.org/obo/AAO_",
        "ADO": "http://purl.obolibrary.org/obo/ADO_",
        "ADW": "https://animaldiversity.org/accounts/",
        "AEO": "http://purl.obolibrary.org/obo/AEO_",
        "AERO": "http://purl.obolibrary.org/obo/AERO_",
        "AGRO": "http://purl.obolibrary.org/obo/AGRO_",
        "AISM": "http://purl.obolibrary.org/obo/AISM_",
        "AMPHX": "http://purl.obolibrary.org/obo/AMPHX_",
        "APO": "http://purl.obolibrary.org/obo/APO_",
        "APOLLO_SV": "http://purl.obolibrary.org/obo/APOLLO_SV_",
        "ARO": "http://purl.obolibrary.org/obo/ARO_",
        "ATO": "http://purl.obolibrary.org/obo/ATO_",
        "BCGO": "http://purl.obolibrary.org/obo/BCGO_",
        "BCO": "http://purl.obolibrary.org/obo/BCO_",
        "BFO": "http://purl.obolibrary.org/obo/BFO_",
        "BILA": "http://purl.obolibrary.org/obo/BILA_",
        "BOOTSTREP": "http://purl.obolibrary.org/obo/BOOTSTREP_",
        "BSPO": "http://purl.obolibrary.org/obo/BSPO_",
        "BTO": "http://purl.obolibrary.org/obo/BTO_",
        "BiDO": "http://purl.org/spar/bido/",
        "BiRO": "http://purl.org/spar/biro/",
        "C4O": "http://purl.org/spar/c4o/",
        "CARO": "http://purl.obolibrary.org/obo/CARO_",
        "CDAO": "http://purl.obolibrary.org/obo/CDAO_",
        "CDNO": "http://purl.obolibrary.org/obo/CDNO_",
        "CEPH": "http://purl.obolibrary.org/obo/CEPH_",
        "CHEBI": "http://purl.obolibrary.org/obo/CHEBI_",
        "CHEMINF": "http://purl.obolibrary.org/obo/CHEMINF_",
        "CHIRO": "http://purl.obolibrary.org/obo/CHIRO_",
        "CHMO": "http://purl.obolibrary.org/obo/CHMO_",
        "CIDO": "http://purl.obolibrary.org/obo/CIDO_",
        "CIO": "http://purl.obolibrary.org/obo/CIO_",
        "CL": "http://purl.obolibrary.org/obo/CL_",
        "CLAO": "http://purl.obolibrary.org/obo/CLAO_",
        "CLO": "http://purl.obolibrary.org/obo/CLO_",
        "CLYH": "http://purl.obolibrary.org/obo/CLYH_",
        "CMF": "http://purl.obolibrary.org/obo/CMF_",
        "CMO": "http://purl.obolibrary.org/obo/CMO_",
        "COB": "http://purl.obolibrary.org/obo/COB_",
        "COLAO": "http://purl.obolibrary.org/obo/COLAO_",
        "CRO": "http://purl.obolibrary.org/obo/CRO_",
        "CTENO": "http://purl.obolibrary.org/obo/CTENO_",
        "CTO": "http://purl.obolibrary.org/obo/CTO_",
        "CVDO": "http://purl.obolibrary.org/obo/CVDO_",
        "CiTO": "http://purl.org/spar/cito/",
        "DC_CL": "http://purl.obolibrary.org/obo/DC_CL_",
        "DDANAT": "http://purl.obolibrary.org/obo/DDANAT_",
        "DDPHENO": "http://purl.obolibrary.org/obo/DDPHENO_",
        "DEO": "http://purl.org/spar/deo/",
        "DIDEO": "http://purl.obolibrary.org/obo/DIDEO_",
        "DINTO": "http://purl.obolibrary.org/obo/DINTO_",
        "DISDRIV": "http://purl.obolibrary.org/obo/DISDRIV_",
        "DOI": "https://doi.org/",
        "DOID": "http://purl.obolibrary.org/obo/DOID_",
        "DPO": "http://purl.obolibrary.org/obo/FBcv_",
        "DRON": "http://purl.obolibrary.org/obo/DRON_",
        "DUO": "http://purl.obolibrary.org/obo/DUO_",
        "DataCite": "http://purl.org/spar/datacite/",
        "DeBiO": "https://biopragmatics.github.io/debio/",
        "DoCO": "http://purl.org/spar/doco/",
        "ECAO": "http://purl.obolibrary.org/obo/ECAO_",
        "ECO": "http://purl.obolibrary.org/obo/ECO_",
        "ECOCORE": "http://purl.obolibrary.org/obo/ECOCORE_",
        "ECTO": "http://purl.obolibrary.org/obo/ECTO_",
        "EFO": "http://www.ebi.ac.uk/efo/EFO_",
        "EHDA": "http://purl.obolibrary.org/obo/EHDA_",
        "EHDAA": "http://purl.obolibrary.org/obo/EHDAA_",
        "EHDAA2": "http://purl.obolibrary.org/obo/EHDAA2_",
        "EMAP": "http://purl.obolibrary.org/obo/EMAP_",
        "EMAPA": "http://purl.obolibrary.org/obo/EMAPA_",
        "ENSEMBL": "https://www.ensembl.org/id/",
        "ENVO": "http://purl.obolibrary.org/obo/ENVO_",
        "EO": "http://archive.gramene.org/db/ontology/search?query=EO:",
        "EPIO": "http://purl.obolibrary.org/obo/EPIO_",
        "EPO": "http://purl.obolibrary.org/obo/EPO_",
        "ERO": "http://purl.obolibrary.org/obo/ERO_",
        "EUPATH": "http://purl.obolibrary.org/obo/EUPATH_",
        "EV": "http://purl.obolibrary.org/obo/EV_",
        "ExO": "http://purl.obolibrary.org/obo/ExO_",
        "FAO": "http://purl.obolibrary.org/obo/FAO_",
        "FBSP": "http://purl.obolibrary.org/obo/FBSP_",
        "FBbi": "http://purl.obolibrary.org/obo/FBbi_",
        "FBbt": "http://purl.obolibrary.org/obo/FBbt_",
        "FBcv": "http://purl.obolibrary.org/obo/FBcv_",
        "FBdv": "http://purl.obolibrary.org/obo/FBdv_",
        "FIDEO": "http://purl.obolibrary.org/obo/FIDEO_",
        "FIX": "http://purl.obolibrary.org/obo/FIX_",
        "FLOPO": "http://purl.obolibrary.org/obo/FLOPO_",
        "FLU": "http://purl.obolibrary.org/obo/FLU_",
        "FMA": "http://purl.obolibrary.org/obo/FMA_",
        "FOBI": "http://purl.obolibrary.org/obo/FOBI_",
        "FOODON": "http://purl.obolibrary.org/obo/FOODON_",
        "FOVT": "http://purl.obolibrary.org/obo/FOVT_",
        "FRAPO": "http://purl.org/cerif/frapo/",
        "FRBR": "http://purl.org/vocab/frbr/core#",
        "FYPO": "http://purl.obolibrary.org/obo/FYPO_",
        "FlyBase": "https://flybase.org/reports/",
        "GAZ": "http://purl.obolibrary.org/obo/GAZ_",
        "GECKO": "http://purl.obolibrary.org/obo/GECKO_",
        "GENEPIO": "http://purl.obolibrary.org/obo/GENEPIO_",
        "GENO": "http://purl.obolibrary.org/obo/GENO_",
        "GEOGEO": "http://purl.obolibrary.org/obo/GEO_",
        "GNO": "http://purl.obolibrary.org/obo/GNO_",
        "GO": "http://purl.obolibrary.org/obo/GO_",
        "GSSO": "http://purl.obolibrary.org/obo/GSSO_",
        "HABRONATTUS": "http://purl.obolibrary.org/obo/HABRONATTUS_",
        "HANCESTRO": "http://purl.obolibrary.org/obo/HANCESTRO_",
        "HAO": "http://purl.obolibrary.org/obo/HAO_",
        "HOM": "http://purl.obolibrary.org/obo/HOM_",
        "HP": "http://purl.obolibrary.org/obo/HP_",
        "HSO": "http://purl.obolibrary.org/obo/HSO_",
        "HTN": "http://purl.obolibrary.org/obo/HTN_",
        "HsapDv": "http://purl.obolibrary.org/obo/HsapDv_",
        "IAO": "http://purl.obolibrary.org/obo/IAO_",
        "ICEO": "http://purl.obolibrary.org/obo/ICEO_",
        "ICO": "http://purl.obolibrary.org/obo/ICO_",
        "IDO": "http://purl.obolibrary.org/obo/IDO_",
        "IDOMAL": "http://purl.obolibrary.org/obo/IDOMAL_",
        "IEV": "http://purl.obolibrary.org/obo/IEV_",
        "IMR": "http://purl.obolibrary.org/obo/IMR_",
        "INO": "http://purl.obolibrary.org/obo/INO_",
        "KISAO": "http://purl.obolibrary.org/obo/KISAO_",
        "LABO": "http://purl.obolibrary.org/obo/LABO_",
        "LEPAO": "http://purl.obolibrary.org/obo/LEPAO_",
        "LIPRO": "http://purl.obolibrary.org/obo/LIPRO_",
        "LOGGERHEAD": "http://purl.obolibrary.org/obo/LOGGERHEAD_",
        "MA": "http://purl.obolibrary.org/obo/MA_",
        "MAMO": "http://purl.obolibrary.org/obo/MAMO_",
        "MAO": "http://purl.obolibrary.org/obo/MAO_",
        "MAT": "http://purl.obolibrary.org/obo/MAT_",
        "MAXO": "http://purl.obolibrary.org/obo/MAXO_",
        "MCO": "http://purl.obolibrary.org/obo/MCO_",
        "MF": "http://purl.obolibrary.org/obo/MF_",
        "MFMO": "http://purl.obolibrary.org/obo/MFMO_",
        "MFO": "http://purl.obolibrary.org/obo/MFO_",
        "MFOEM": "http://purl.obolibrary.org/obo/MFOEM_",
        "MFOMD": "http://purl.obolibrary.org/obo/MFOMD_",
        "MGI": "http://www.informatics.jax.org/accession/MGI:",
        "MI": "http://purl.obolibrary.org/obo/MI_",
        "MIAPA": "http://purl.obolibrary.org/obo/MIAPA_",
        "MICRO": "http://purl.obolibrary.org/obo/MICRO_",
        "MIRNAO": "http://purl.obolibrary.org/obo/MIRNAO_",
        "MIRO": "http://purl.obolibrary.org/obo/MIRO_",
        "MMO": "http://purl.obolibrary.org/obo/MMO_",
        "MO": "http://purl.obolibrary.org/obo/MO_",
        "MOD": "http://purl.obolibrary.org/obo/MOD_",
        "MONDO": "http://purl.obolibrary.org/obo/MONDO_",
        "MOP": "http://purl.obolibrary.org/obo/MOP_",
        "MP": "http://purl.obolibrary.org/obo/MP_",
        "MPATH": "http://purl.obolibrary.org/obo/MPATH_",
        "MPIO": "http://purl.obolibrary.org/obo/MPIO_",
        "MRO": "http://purl.obolibrary.org/obo/MRO_",
        "MS": "http://purl.obolibrary.org/obo/MS_",
        "MmusDv": "http://purl.obolibrary.org/obo/MmusDv_",
        "NBO": "http://purl.obolibrary.org/obo/NBO_",
        "NCBIGene": "https://www.ncbi.nlm.nih.gov/gene/",
        "NCBIProtein": "https://www.ncbi.nlm.nih.gov/protein/",
        "NCBITaxon": "http://purl.obolibrary.org/obo/NCBITaxon_",
        "NCIT": "http://purl.obolibrary.org/obo/NCIT_",
        "NCRO": "http://purl.obolibrary.org/obo/NCRO_",
        "NIF_CELL": "http://purl.obolibrary.org/obo/NIF_CELL_",
        "NIF_DYSFUNCTION": "http://purl.obolibrary.org/obo/NIF_DYSFUNCTION_",
        "NIF_GROSSANATOMY": "http://purl.obolibrary.org/obo/NIF_GROSSANATOMY_",
        "NMR": "https://www.ebi.ac.uk/ols/ontologies/nmrcv/terms?short_form=NMR:",
        "NOMEN": "http://purl.obolibrary.org/obo/NOMEN_",
        "OAE": "http://purl.obolibrary.org/obo/OAE_",
        "OARCS": "http://purl.obolibrary.org/obo/OARCS_",
        "OBA": "http://purl.obolibrary.org/obo/OBA_",
        "OBCS": "http://purl.obolibrary.org/obo/OBCS_",
        "OBI": "http://purl.obolibrary.org/obo/OBI_",
        "OBIB": "http://purl.obolibrary.org/obo/OBIB_",
        "OGG": "http://purl.obolibrary.org/obo/OGG_",
        "OGI": "http://purl.obolibrary.org/obo/OGI_",
        "OGMS": "http://purl.obolibrary.org/obo/OGMS_",
        "OGSF": "http://purl.obolibrary.org/obo/OGSF_",
        "OHD": "http://purl.obolibrary.org/obo/OHD_",
        "OHMI": "http://purl.obolibrary.org/obo/OHMI_",
        "OHPI": "http://purl.obolibrary.org/obo/OHPI_",
        "OMIABIS": "http://purl.obolibrary.org/obo/OMIABIS_",
        "OMIT": "http://purl.obolibrary.org/obo/OMIT_",
        "OMO": "http://purl.obolibrary.org/obo/OMO_",
        "OMP": "http://purl.obolibrary.org/obo/OMP_",
        "OMRSE": "http://purl.obolibrary.org/obo/OMRSE_",
        "ONE": "http://purl.obolibrary.org/obo/ONE_",
        "ONS": "http://purl.obolibrary.org/obo/ONS_",
        "ONTOAVIDA": "http://purl.obolibrary.org/obo/ONTOAVIDA_",
        "ONTONEO": "http://purl.obolibrary.org/obo/ONTONEO_",
        "OOSTT": "http://purl.obolibrary.org/obo/OOSTT_",
        "OPL": "http://purl.obolibrary.org/obo/OPL_",
        "OPMI": "http://purl.obolibrary.org/obo/OPMI_",
        "ORNASEQ": "http://purl.obolibrary.org/obo/ORNASEQ_",
        "OVAE": "http://purl.obolibrary.org/obo/OVAE_",
        "OlatDv": "http://purl.obolibrary.org/obo/OlatDv_",
        "PAO": "http://purl.obolibrary.org/obo/PAO_",
        "PATO": "http://purl.obolibrary.org/obo/PATO_",
        "PCL": "http://purl.obolibrary.org/obo/PCL_",
        "PCO": "http://purl.obolibrary.org/obo/PCO_",
        "PDRO": "http://purl.obolibrary.org/obo/PDRO_",
        "PD_ST": "http://purl.obolibrary.org/obo/PD_ST_",
        "PECO": "http://purl.obolibrary.org/obo/PECO_",
        "PGDSO": "http://purl.obolibrary.org/obo/PGDSO_",
        "PHIPO": "http://purl.obolibrary.org/obo/PHIPO_",
        "PLANA": "http://purl.obolibrary.org/obo/PLANA_",
        "PLANP": "http://purl.obolibrary.org/obo/PLANP_",
        "PLO": "http://purl.obolibrary.org/obo/PLO_",
        "PO": "http://purl.obolibrary.org/obo/PO_",
        "PORO": "http://purl.obolibrary.org/obo/PORO_",
        "PPO": "http://purl.obolibrary.org/obo/PPO_",
        "PR": "http://purl.obolibrary.org/obo/PR_",
        "PROCO": "http://purl.obolibrary.org/obo/PROCO_",
        "PROPREO": "http://purl.obolibrary.org/obo/PROPREO_",
        "PSDO": "http://purl.obolibrary.org/obo/PSDO_",
        "PSO": "http://purl.obolibrary.org/obo/PSO_",
        "PW": "http://purl.obolibrary.org/obo/PW_",
        "PWO": "http://purl.org/spar/pwo/",
        "PdumDv": "http://purl.obolibrary.org/obo/PdumDv_",
        "PuRO": "http://purl.org/spar/pro/",
        "RBO": "http://purl.obolibrary.org/obo/RBO_",
        "RESID": "https://proteininformationresource.org/cgi-bin/resid?id=",
        "REX": "http://purl.obolibrary.org/obo/REX_",
        "RNAO": "http://purl.obolibrary.org/obo/RNAO_",
        "RO": "http://purl.obolibrary.org/obo/RO_",
        "RS": "http://purl.obolibrary.org/obo/RS_",
        "RXNO": "http://purl.obolibrary.org/obo/RXNO_",
        "SAO": "http://uri.neuinfo.org/nif/nifstd/sao",
        "SBO": "http://purl.obolibrary.org/obo/SBO_",
        "SCDO": "http://purl.obolibrary.org/obo/SCDO_",
        "SCTID": "http://www.snomedbrowser.com/Codes/Details/",
        "SCoRO": "http://purl.org/spar/scoro/",
        "SEP": "http://purl.obolibrary.org/obo/SEP_",
        "SEPIO": "http://purl.obolibrary.org/obo/SEPIO_",
        "SIBO": "http://purl.obolibrary.org/obo/SIBO_",
        "SO": "http://purl.obolibrary.org/obo/SO_",
        "SOPHARM": "http://purl.obolibrary.org/obo/SOPHARM_",
        "SPD": "http://purl.obolibrary.org/obo/SPD_",
        "STATO": "http://purl.obolibrary.org/obo/STATO_",
        "SWO": "http://purl.obolibrary.org/obo/SWO_",
        "SYMP": "http://purl.obolibrary.org/obo/SYMP_",
        "TADS": "http://purl.obolibrary.org/obo/TADS_",
        "TAHE": "http://purl.obolibrary.org/obo/TAHE_",
        "TAHH": "http://purl.obolibrary.org/obo/TAHH_",
        "TAO": "http://purl.obolibrary.org/obo/TAO_",
        "TAXRANK": "http://purl.obolibrary.org/obo/TAXRANK_",
        "TGMA": "http://purl.obolibrary.org/obo/TGMA_",
        "TO": "http://purl.obolibrary.org/obo/TO_",
        "TRANS": "http://purl.obolibrary.org/obo/TRANS_",
        "TTO": "http://purl.obolibrary.org/obo/TTO_",
        "TXPO": "http://purl.obolibrary.org/obo/TXPO_",
        "UBERON": "http://purl.obolibrary.org/obo/UBERON_",
        "UMLS": "http://linkedlifedata.com/resource/umls/id/",
        "UO": "http://purl.obolibrary.org/obo/UO_",
        "UPA": "http://www.grenoble.prabi.fr/obiwarehouse/unipathway/upa?upid=",
        "UPHENO": "http://purl.obolibrary.org/obo/UPHENO_",
        "VHOG": "http://purl.obolibrary.org/obo/VHOG_",
        "VO": "http://purl.obolibrary.org/obo/VO_",
        "VSAO": "http://purl.obolibrary.org/obo/VSAO_",
        "VT": "http://purl.obolibrary.org/obo/VT_",
        "VTO": "http://purl.obolibrary.org/obo/VTO_",
        "VariO": "http://purl.obolibrary.org/obo/VariO_",
        "WBPhenotype": "http://purl.obolibrary.org/obo/WBPhenotype_",
        "WBbt": "http://purl.obolibrary.org/obo/WBbt_",
        "WBls": "http://purl.obolibrary.org/obo/WBls_",
        "WormBase": "https://www.wormbase.org/get?name=",
        "XAO": "http://purl.obolibrary.org/obo/XAO_",
        "XCO": "http://purl.obolibrary.org/obo/XCO_",
        "XLMOD": "http://purl.obolibrary.org/obo/XLMOD_",
        "XPO": "http://purl.obolibrary.org/obo/XPO_",
        "ZEA": "http://purl.obolibrary.org/obo/ZEA_",
        "ZECO": "http://purl.obolibrary.org/obo/ZECO_",
        "ZFA": "http://purl.obolibrary.org/obo/ZFA_",
        "ZFS": "http://purl.obolibrary.org/obo/ZFS_",
        "ZP": "http://purl.obolibrary.org/obo/ZP_",
        "abcd": "https://web.expasy.org/abcd/ABCD_",
        "abm": "https://www.abmgood.com/catalogsearch/result/?cat=&q=",
        "ac": "http://www.w3.org/ns/activitystreams#",
        "aceview.worm": "https://www.ncbi.nlm.nih.gov/IEB/Research/Acembly/av.cgi?db=worm&c=Gene&l=",
        "addexbio": "https://www.addexbio.com/productdetail?pid=",
        "addgene": "http://addgene.org/",
        "affy.probeset": "https://www.affymetrix.com/LinkServlet?probeset=",
        "aftol.taxonomy": "http://wasabi.lutzonilab.net/pub/displayTaxonInfo?aftol_id=",
        "agricola": "https://agricola.nal.usda.gov/vwebv/holdingsInfo?bibId=",
        "agrkb": "https://www.alliancegenome.org/accession/",
        "agrovoc": "http://aims.fao.org/aos/agrovoc/c_",
        "allergome": "http://www.allergome.org/script/dettaglio.php?id_molecule=",
        "alzforum.mutation": "https://www.alzforum.org/mutations/",
        "amoebadb": "https://amoebadb.org/amoeba/app/record/gene/",
        "antibodyregistry": "http://antibodyregistry.org/AB_",
        "antweb": "http://www.antweb.org/specimen.do?name=",
        "aop": "https://aopwiki.org/aops/",
        "aop.events": "https://aopwiki.org/events/",
        "aop.relationships": "https://aopwiki.org/relationships/",
        "aop.stressor": "https://aopwiki.org/stressors/",
        "apaonto": "http://ontology.apa.org/apaonto/termsonlyOUT%20(5).owl#",
        "apd": "http://aps.unmc.edu/AP/database/query_output.php?ID=",
        "aphidbase.transcript": "http://bipaa.genouest.org/apps/grs-2.3/grs?reportID=aphidbase_transcript_report&objectID=",
        "apid.interactions": "http://cicblade.dep.usal.es:8080/APID/Interactions.action?protein=",
        "arachnoserver": "http://www.arachnoserver.org/toxincard.html?id=",
        "araport": "https://www.arabidopsis.org/servlets/TairObject?type=locus&name=",
        "ardb": "http://ardb.cbcb.umd.edu/cgi/search.cgi?db=L&amp;field=ni&amp;term=",
        "ark": "http://n2t.net/ark:",
        "arrayexpress": "https://www.ebi.ac.uk/arrayexpress/experiments/",
        "arrayexpress.platform": "https://www.ebi.ac.uk/arrayexpress/arrays/",
        "arraymap": "https://www.arraymap.org/pgx:",
        "arxiv": "https://arxiv.org/abs/",
        "asap": "http://asap.ahabs.wisc.edu/asap/feature_info.php?LocationID=WIS&FeatureID=",
        "ascl": "http://ascl.net/",
        "asin": "https://amzn.com/",
        "aspgd.locus": "http://www.aspergillusgenome.org/cgi-bin/locus.pl?dbid=",
        "aspgd.protein": "http://www.aspergillusgenome.org/cgi-bin/protein/proteinPage.pl?dbid=",
        "atc": "http://www.whocc.no/atc_ddd_index/?code=",
        "atcc": "https://www.atcc.org/products/",
        "atcvet": "http://www.whocc.no/atcvet/atcvet_index/?code=",
        "atfdb.family": "http://www.bioguo.org/AnimalTFDB/family.php?fam=",
        "autdb": "http://autism.mindspec.org/GeneDetail/",
        "bacdive": "https://bacdive.dsmz.de/strain/",
        "bacmap.biog": "http://bacmap.wishartlab.com/organisms/",
        "bactibase": "http://bactibase.hammamilab.org/",
        "bao": "http://bioportal.bioontology.org/ontologies/BAO/bao:BAO_",
        "bbkg": "https://bbp.epfl.ch/nexus/web/studios/public/",
        "bbtp": "https://bbp.epfl.ch/nexus/web/studios/public/topological-sampling/studios/data:",
        "bcrc": "https://catalog.bcrc.firdi.org.tw/BcrcContent?bid=",
        "bcrj": "http://bcrj.org.br/celula/",
        "bdgp.est": "https://www.ncbi.nlm.nih.gov/nucest/",
        "bdgp.insertion": "http://flypush.imgen.bcm.tmc.edu/pscreen/details.php?line=",
        "bdsc": "https://bdsc.indiana.edu/stocks/",
        "beetlebase": "http://beetlebase.org/cgi-bin/gbrowse/BeetleBase3.gff3/?name=",
        "begdb": "http://www.begdb.com/index.php?action=oneMolecule&state=show&id=",
        "bgee.family": "http://bgee.unil.ch/bgee/bgee?page=gene_family&action=family_details&gene_family_id=",
        "bgee.gene": "https://bgee.org/?page=gene&gene_id=",
        "bgee.organ": "http://bgee.unil.ch/bgee/bgee?page=anatomy&action=organ_details&organ_children=on&organ_id=",
        "bgee.stage": "http://bgee.unil.ch/bgee/bgee?page=anatomy&action=organs&stage_children=on&stage_id=",
        "bigg.compartment": "http://bigg.ucsd.edu/compartments/",
        "bigg.metabolite": "http://bigg.ucsd.edu/models/universal/metabolites/",
        "bigg.model": "http://bigg.ucsd.edu/models/",
        "bigg.reaction": "http://bigg.ucsd.edu/models/universal/reactions/",
        "bindingdb": "http://www.bindingdb.org/compact/",
        "biocarta.pathway": "https://cgap.nci.nih.gov/Pathways/BioCarta/",
        "biocatalogue.service": "https://www.biocatalogue.org/services/",
        "biocyc": "http://biocyc.org/getid?id=",
        "biogrid": "http://thebiogrid.org/",
        "biogrid.interaction": "https://thebiogrid.org/interaction/",
        "biolegend": "https://www.biolegend.com/Default.aspx?ID=6664&productid=",
        "biolink": "https://w3id.org/biolink/vocab/",
        "biominder": "https://datalab.rwth-aachen.de/MINDER/resource/",
        "biomodels.db": "https://www.ebi.ac.uk/biomodels/",
        "biomodels.teddy": "https://www.ebi.ac.uk/ols/ontologies/teddy/terms?iri=http://identifiers.org/teddy/TEDDY_",
        "biomodels.vocabulary": "http://biomodels.net/rdf/vocabulary.rdf#",
        "bionumbers": "https://bionumbers.hms.harvard.edu/bionumber.aspx?id=",
        "bioportal": "http://bioportal.bioontology.org/ontologies/",
        "bioproject": "https://www.ncbi.nlm.nih.gov/bioproject/?term=",
        "bioregistry": "https://bioregistry.io/registry/",
        "bioregistry.collection": "https://bioregistry.io/collection/",
        "bioregistry.registry": "https://bioregistry.io/metaregistry/",
        "bioregistry.schema": "https://bioregistry.io/schema/#",
        "biorxiv": "https://www.biorxiv.org/content/10.1101/",
        "biosample": "https://www.ebi.ac.uk/biosamples/sample/",
        "biosimulations": "https://biosimulations.org/projects/",
        "biosimulators": "https://biosimulators.org/simulators/",
        "biostudies": "https://www.ebi.ac.uk/biostudies/studies/",
        "biosystems": "https://www.ncbi.nlm.nih.gov/biosystems/",
        "biotools": "https://bio.tools/",
        "biozil": "https://www.biozol.de/en/product/",
        "birnlex": "http://uri.neuinfo.org/nif/nifstd/birnlex_",
        "bitbucket": "https://bitbucket.org/",
        "bitterdb.cpd": "http://bitterdb.agri.huji.ac.il/bitterdb/compound.php?id=",
        "bitterdb.rec": "http://bitterdb.agri.huji.ac.il/Receptor.php?id=",
        "bko": "http://www.sbgnbricks.org/BKO/full/entry/all/BKO:",
        "bmrb.restraint": "https://restraintsgrid.bmrb.io/NRG/MRGridServlet?block_text_type=2-parsed&db_username=wattos1&file_detail=2-parsed&format=n%2Fa&program=STAR&request_type=block&subtype=full&type=entry&mrblock_id=",
        "bold.taxonomy": "http://www.boldsystems.org/index.php/Taxbrowser_Taxonpage?taxid=",
        "brenda": "https://www.brenda-enzymes.org/php/result_flat.php4?ecno=",
        "brenda.ligand": "https://www.brenda-enzymes.de/ligand.php?brenda_ligand_id=",
        "brenda.ligandgroup": "https://www.brenda-enzymes.de/ligand.php?brenda_group_id=",
        "broad": "https://www.broadinstitute.org/annotation/genome/magnaporthe_grisea/GeneDetails.html?sp=",
        "cameo": "https://www.cameo3d.org/sp/targets/target/",
        "caps": "http://www.bioinsilico.org/cgi-bin/CAPSDB/getCAPScluster?nidcl=",
        "cas": "https://commonchemistry.cas.org/detail?ref=",
        "casspc": "https://researcharchive.calacademy.org/research/ichthyology/catalog/fishcatget.asp?spid=",
        "cath": "http://www.cathdb.info/cathnode/",
        "cath.domain": "http://www.cathdb.info/domain/",
        "cath.superfamily": "http://www.cathdb.info/cathnode/",
        "cattleqtldb": "https://www.animalgenome.org/QTLdb/q?id=QTL_ID:",
        "cba": "https://europepmc.org/article/CBA/",
        "cbioportal": "https://www.cbioportal.org/study/summary?id=",
        "ccds": "http://www.ncbi.nlm.nih.gov/CCDS/CcdsBrowse.cgi?REQUEST=CCDS&DATA=",
        "ccle": "https://www.cbioportal.org/patient?studyId=ccle_broad_2019&caseId=",
        "cco": "https://www.ebi.ac.uk/ols/ontologies/cco/terms?obo_id=CCO:",
        "ccrid": "http://www.cellresource.cn/cellsearch.aspx?sc=1&where=",
        "cdd": "https://www.ncbi.nlm.nih.gov/Structure/cdd/cddsrv.cgi?uid=",
        "cdpd": "http://webprod3.hc-sc.gc.ca/dpd-bdpp/info.do?lang=eng&code=",
        "cell_biolabs": "https://www.cellbiolabs.com/search?keywords=",
        "cell_model_passport": "https://cellmodelpassports.sanger.ac.uk/passports/",
        "cellimage": "http://cellimagelibrary.org/images/",
        "cellopub": "https://web.expasy.org/cellosaurus/cellopub/",
        "cellosaurus": "https://web.expasy.org/cellosaurus/CVCL_",
        "cellosaurus.resource": "https://bioregistry.io/metaregistry/cellosaurus/",
        "cellrepo": "https://cellrepo.ico2s.org/repositories/",
        "cgd": "http://www.candidagenome.org/cgi-bin/locus.pl?dbid=",
        "cgnc": "http://birdgenenames.org/cgnc/GeneReport?id=",
        "cgsc": "http://cgsc.biology.yale.edu/Site.php?ID=",
        "charprot": "http://www.jcvi.org/charprotdb/index.cgi/view/",
        "chembl": "https://www.ebi.ac.uk/chembl/entity/",
        "chembl.cell": "https://www.ebi.ac.uk/chembl/cell_line_report_card/",
        "chembl.compound": "https://www.ebi.ac.uk/chembl/compound/inspect/",
        "chembl.target": "https://www.ebi.ac.uk/chembl/target/inspect/",
        "chemdb": "http://cdb.ics.uci.edu/cgibin/ChemicalDetailWeb.py?chemical_id=",
        "chemidplus": "https://chem.nlm.nih.gov/chemidplus/rn/",
        "chickenqtldb": "https://www.animalgenome.org/QTLdb/q?id=QTL_ID:",
        "citexplore": "https://europepmc.org/article/CTX/",
        "civic.aid": "https://civicdb.org/links/assertions/",
        "civic.eid": "https://civicdb.org/links/evidence/",
        "classyfire": "http://classyfire.wishartlab.com/tax_nodes/C",
        "clingene": "https://reg.clinicalgenome.org/redmine/projects/registry/genboree_registry/by_caid?caid=",
        "clinicaltrials": "https://clinicaltrials.gov/ct2/show/",
        "clinvar": "https://www.ncbi.nlm.nih.gov/clinvar/variation/",
        "clinvar.record": "http://www.ncbi.nlm.nih.gov/clinvar/",
        "clinvar.submission": "http://www.ncbi.nlm.nih.gov/clinvar?term=",
        "clinvar.submitter": "https://www.ncbi.nlm.nih.gov/clinvar/submitters/",
        "cls": "https://cls.shop/",
        "co_320": "https://www.cropontology.org/rdf/CO_320:",
        "co_321": "https://www.cropontology.org/rdf/CO_321:",
        "co_322": "https://www.cropontology.org/rdf/CO_322:",
        "co_323": "https://www.cropontology.org/rdf/CO_323:",
        "co_324": "https://www.cropontology.org/rdf/CO_324:",
        "co_325": "https://www.cropontology.org/rdf/CO_325:",
        "co_327": "https://www.cropontology.org/rdf/CO_327:",
        "co_330": "https://www.cropontology.org/rdf/CO_330:",
        "co_331": "https://www.cropontology.org/rdf/CO_331:",
        "co_333": "https://www.cropontology.org/rdf/CO_333:",
        "co_334": "https://www.cropontology.org/rdf/CO_334:",
        "co_335": "https://www.cropontology.org/rdf/CO_335:",
        "co_336": "https://www.cropontology.org/rdf/CO_336:",
        "co_337": "https://www.cropontology.org/rdf/CO_337:",
        "co_338": "https://www.cropontology.org/rdf/CO_338:",
        "co_339": "https://www.cropontology.org/rdf/CO_339:",
        "co_340": "https://www.cropontology.org/rdf/CO_340:",
        "co_341": "https://www.cropontology.org/rdf/CO_341:",
        "co_343": "https://www.cropontology.org/rdf/CO_343:",
        "co_345": "https://www.cropontology.org/rdf/CO_345:",
        "co_346": "https://www.cropontology.org/rdf/CO_346:",
        "co_347": "https://www.cropontology.org/rdf/CO_347:",
        "co_348": "https://www.cropontology.org/rdf/CO_348:",
        "co_350": "https://www.cropontology.org/rdf/CO_350:",
        "co_356": "https://www.cropontology.org/rdf/CO_356:",
        "co_357": "https://www.cropontology.org/rdf/CO_357:",
        "co_358": "https://www.cropontology.org/rdf/CO_358:",
        "co_359": "https://www.cropontology.org/rdf/CO_359:",
        "co_360": "https://www.cropontology.org/rdf/CO_360:",
        "co_365": "https://www.cropontology.org/rdf/CO_365:",
        "co_366": "https://www.cropontology.org/rdf/CO_366:",
        "coconut": "https://coconut.naturalproducts.net/compound/coconut_id/",
        "cog": "https://www.ncbi.nlm.nih.gov/research/cog/cog/",
        "cog.category": "https://www.ncbi.nlm.nih.gov/research/cog/cogcategory/",
        "cog.pathway": "https://www.ncbi.nlm.nih.gov/research/cog/pathway/",
        "col.taiwan": "https://taibnet.sinica.edu.tw/eng/taibnet_species_detail.php?name_code=",
        "colonatlas": "http://www.coloncanceratlas.org/search_cell_line?cell_line=",
        "combine.specifications": "https://co.mbine.org/specifications/",
        "come": "https://www.flymine.org/come/entry?gn=",
        "complexportal": "https://www.ebi.ac.uk/complexportal/complex/",
        "comptox": "https://comptox.epa.gov/dashboard/",
        "compulyeast": "http://compluyeast2dpage.dacya.ucm.es/cgi-bin/2d/2d.cgi?ac=",
        "conoserver": "http://www.conoserver.org/?page=card&table=protein&id=",
        "conso": "https://pharmacome.github.io/conso/",
        "cordis.article": "https://cordis.europa.eu/article/id/",
        "cordis.project": "https://cordis.europa.eu/project/id/",
        "coriell": "http://ccr.coriell.org/Sections/Search/Sample_Detail.aspx?Ref=",
        "corrdb": "https://www.animalgenome.org/CorrDB/q/?id=CorrID:",
        "corum": "https://mips.helmholtz-muenchen.de/corum/?id=",
        "cosmic": "http://cancer.sanger.ac.uk/cosmic/gene/overview?ln=",
        "cosmic.cell": "https://cancer.sanger.ac.uk/cell_lines/sample/overview?id=",
        "covid19": "https://covid19.sfb.uit.no/api/records/",
        "cpc": "https://worldwide.espacenet.com/classification?locale=en_EP#!/CPC=",
        "cpt": "https://www.aapc.com/codes/cpt-codes/",
        "crisprdb": "http://crispr.i2bc.paris-saclay.fr/cgi-bin/crispr/SpecieProperties_db.cgi?Taxon_id[]=",
        "cryptodb": "https://cryptodb.org/cryptodb/app/record/gene/",
        "csa": "https://www.ebi.ac.uk/thornton-srv/databases/CSA/SearchResults.php?PDBID=",
        "ctd.chemical": "http://ctdbase.org/detail.go?type=chem&acc=",
        "ctd.disease": "http://ctdbase.org/detail.go?type=disease&db=MESH&acc=",
        "ctd.gene": "http://ctdbase.org/detail.go?type=gene&acc=",
        "dailymed": "https://dailymed.nlm.nih.gov/dailymed/drugInfo.cfm?setid=",
        "dandi": "https://dandiarchive.org/dandiset/",
        "darc": "http://darcsite.genzentrum.lmu.de/darc/view.php?id=",
        "dashr": "http://lisanwanglab.org/DASHR/entry/",
        "datanator.gene": "https://www.datanator.info/gene/",
        "datanator.metabolite": "https://www.datanator.info/metabolite/",
        "datanator.reaction": "https://datanator.info/reaction/",
        "datf": "http://planttfdb.cbi.pku.edu.cn/tf.php?sp=Ath&did=",
        "dbest": "https://www.ncbi.nlm.nih.gov/nucest/",
        "dbg2introns": "http://webapps2.ucalgary.ca/~groupii/cgi-bin/intron.cgi?name=",
        "dbgap": "https://www.ncbi.nlm.nih.gov/projects/gap/cgi-bin/study.cgi?study_id=",
        "dbprobe": "https://www.ncbi.nlm.nih.gov/probe/?term=",
        "dbsnp": "https://www.ncbi.nlm.nih.gov/snp/",
        "dbvar.study": "https://www.ncbi.nlm.nih.gov/dbvar/studies/",
        "dbvar.variant": "https://www.ncbi.nlm.nih.gov/dbvar/variants/",
        "dc": "http://purl.org/dc/terms/",
        "dcat": "http://www.w3.org/ns/dcat#",
        "dcterms": "http://purl.org/dc/terms/",
        "dctypes": "http://purl.org/dc/dcmitype/",
        "ddinter.drug": "http://ddinter.scbdd.com/ddinter/drug-detail/",
        "ddinter.interaction": "http://ddinter.scbdd.com/ddinter/interact/",
        "decipher": "https://www.deciphergenomics.org/syndrome/",
        "degradome": "http://degradome.uniovi.es/cgi-bin/protease/",
        "depmap": "https://depmap.org/portal/cell_line/",
        "depod": "http://www.depod.bioss.uni-freiburg.de/showp.php?gene=",
        "dev.ga4ghdos": "https://dos-gdc.ucsc-cgp-dev.org/ga4gh/dos/v1/dataobjects/",
        "dg.4503": "https://gen3.biodatacatalyst.nhlbi.nih.gov/ga4gh/drs/v1/objects/",
        "dg.4dfc": "https://nci-crdc.datacommons.io/ga4gh/drs/v1/objects/",
        "dg.6vts": "https://jcoin.datacommons.io/ga4gh/drs/v1/objects/",
        "dg.anv0": "https://gen3.theanvil.io/ga4gh/drs/v1/objects/",
        "dg.f82a1a": "https://data.kidsfirstdrc.org/ga4gh/drs/v1/objects/",
        "dg5b0d": "https://data.bloodpac.org/ga4gh/drs/v1/objects/",
        "dgrc": "https://dgrc.bio.indiana.edu/product/View?product=",
        "dictybase": "http://dictybase.org/gene/",
        "dictybase.est": "http://dictybase.org/db/cgi-bin/feature_page.pl?primary_id=",
        "dictybase.gene": "http://dictybase.org/gene/",
        "did": "https://uniresolver.io/#did:",
        "dip": "https://dip.doe-mbi.ucla.edu/dip/DIPview.cgi?ID=",
        "disprot": "https://disprot.org/",
        "disprot.region": "https://www.disprot.org/",
        "dlxb": "https://doulix.com/biomodules/",
        "dlxc": "https://doulix.com/constructs/",
        "dommino": "http://orion.rnet.missouri.edu/~nz953/DOMMINO/index.php/result/show_network/",
        "door": "http://csbl.bmb.uga.edu/DOOR/operon.php?id=",
        "doqcs.model": "http://doqcs.ncbs.res.in/template.php?&y=accessiondetails&an=",
        "doqcs.pathway": "http://doqcs.ncbs.res.in/template.php?&y=pathwaydetails&pn=",
        "dpv": "http://www.dpvweb.net/dpv/showdpv.php?dpvno=",
        "drsc": "http://www.flyrnai.org/cgi-bin/RNAi_gene_lookup_public.pl?gname=",
        "drugbank": "http://www.drugbank.ca/drugs/",
        "drugbank.category": "https://www.drugbank.ca/categories/",
        "drugbank.salt": "https://go.drugbank.com/salts/",
        "drugbankv4.target": "http://www.drugbank.ca/biodb/bio_entities/",
        "drugcentral": "http://drugcentral.org/drugcard/",
        "dsmz": "https://www.dsmz.de/collection/catalogue/details/culture/",
        "eaglei": "http://hawaii.eagle-i.net/i/",
        "easychair.cfp": "https://easychair.org/cfp/",
        "easychair.topic": "https://easychair.org/cfp/topic.cgi?tid=",
        "ebisc": "https://cells.ebisc.org/",
        "ecacc": "https://www.phe-culturecollections.org.uk/products/celllines/generalcell/detail.jsp?collection=ecacc_gc&refId=",
        "eccode": "https://www.ebi.ac.uk/intenz/query?cmd=SearchEC&ec=",
        "echobase": "http://www.york.ac.uk/res/thomas/Gene.cfm?recordID=",
        "ecmdb": "http://ecmdb.ca/compounds/",
        "ecocyc": "https://biocyc.org/gene?id=",
        "ecogene": "http://www.ecogene.org/gene/",
        "ecyano.entity": "https://www.e-cyanobacterium.org/bcs/entity/",
        "ecyano.experiment": "https://www.e-cyanobacterium.org/experiments-repository/?action=detail&id=",
        "ecyano.model": "https://e-cyanobacterium.org/models/model/",
        "ecyano.rule": "https://e-cyanobacterium.org/bcs/rule/",
        "edam": "https://www.ebi.ac.uk/ols/ontologies/edam/terms?iri=http://edamontology.org/",
        "edam.data": "http://edamontology.org/data_",
        "edam.format": "http://edamontology.org/format_",
        "edam.operation": "http://edamontology.org/operation_",
        "edam.topic": "http://edamontology.org/topic_",
        "ega.dataset": "https://www.ebi.ac.uk/ega/datasets/",
        "ega.study": "https://www.ebi.ac.uk/ega/studies/",
        "eggnog": "http://eggnog.embl.de/version_3.0/cgi/search.py?search_term_0=",
        "emdb": "https://www.ebi.ac.uk/pdbe/entry/emdb/",
        "emolecules": "https://reaxys.emolecules.com/cgi-bin/more?vid=",
        "empiar": "https://www.ebi.ac.uk/pdbe/emdb/empiar/entry/",
        "ena.embl": "https://www.ebi.ac.uk/ena/browser/view/",
        "encode": "https://www.encodeproject.org/",
        "ensembl.bacteria": "https://bacteria.ensembl.org/id/",
        "ensembl.fungi": "https://fungi.ensembl.org/id/",
        "ensembl.metazoa": "https://metazoa.ensembl.org/id/",
        "ensembl.plant": "https://plants.ensembl.org/id/",
        "ensembl.protist": "https://protists.ensembl.org/id/",
        "envipath": "https://envipath.org/package/",
        "enzo": "https://www.enzolifesciences.com/",
        "eolife": "https://eol.org/pages/",
        "epd": "http://epd.vital-it.ch/cgi-bin/query_result.pl?out_format=NICE&Entry_0=",
        "erm": "https://nanocommons.github.io/identifiers/registry#",
        "erv": "https://herv.img.cas.cz/s/",
        "estdab": "https://www.ebi.ac.uk/cgi-bin/ipd/estdab/print_cell.cgi?ESTDAB-",
        "eu89h": "http://data.europa.eu/89h/",
        "euclinicaltrials": "https://www.clinicaltrialsregister.eu/ctr-search/search?query=",
        "exac.gene": "http://exac.broadinstitute.org/gene/",
        "exac.transcript": "http://exac.broadinstitute.org/transcript/",
        "exac.variant": "http://exac.broadinstitute.org/variant/",
        "fabio": "https://sparontologies.github.io/fabio/current/fabio.html#",
        "facebase": "https://www.facebase.org/data/record/#1/isa:dataset/accession=",
        "fairsharing": "https://fairsharing.org/",
        "faldo": "http://biohackathon.org/resource/faldo#",
        "fbrf": "https://flybase.org/reports/FBrf",
        "fbtc": "https://flybase.org/reports/FBtc",
        "fcb": "https://w3id.org/faircookbook/",
        "fcsfree": "https://fcs-free.org/fcs-database?",
        "flowrepository": "https://flowrepository.org/id/",
        "foaf": "http://xmlns.com/foaf/0.1/",
        "foodb.compound": "http://foodb.ca/compounds/",
        "foodb.food": "https://foodb.ca/foods/",
        "fossilworks.journal": "http://fossilworks.org/?a=referenceInfo&reference_no=",
        "fossilworks.taxon": "http://www.fossilworks.org/cgi-bin/bridge.pl?a=taxonInfo&taxon_no=",
        "fplx": "https://sorgerlab.github.io/famplex/",
        "fsnp": "http://compbio.cs.queensu.ca/cgi-bin/compbio/search/main.cgi?search_mode=id&amp;id_type=snp_id&amp;id_val=",
        "ftt": "https://apps.usgs.gov/thesaurus/term-simple.php?thcode=3&code=",
        "funcbase.fly": "http://func.mshri.on.ca/fly/genes/list_functional_scores/",
        "funcbase.human": "http://func.mshri.on.ca/human/genes/list_functional_scores/",
        "funcbase.mouse": "http://func.mshri.on.ca/mouse/genes/list_functional_scores/",
        "funcbase.yeast": "http://func.mshri.on.ca/yeast/genes/list_functional_scores/",
        "funderregistry": "http://data.crossref.org/fundingdata/funder/10.13039/",
        "fungidb": "https://fungidb.org/fungidb/app/record/gene/",
        "fungorum": "http://www.indexfungorum.org/names/NamesRecord.asp?RecordID=",
        "ga4ghdos": "https://dataguids.org/ga4gh/dos/v1/dataobjects/",
        "gateway": "https://web.www.healthdatagateway.org/dataset/",
        "gbif": "https://www.gbif.org/species/",
        "gc": "https://www.ncbi.nlm.nih.gov/Taxonomy/taxonomyhome.html/index.cgi?chapter=cgencodes#SG",
        "gcst": "https://www.ebi.ac.uk/gwas/studies/",
        "gdc": "https://portal.gdc.cancer.gov/cases/",
        "gdsc": "https://www.cancerrxgene.org/translation/Drug/",
        "gemet": "https://www.eionet.europa.eu/gemet/en/concept/",
        "genatlas": "http://genatlas.medecine.univ-paris5.fr/fiche.php?symbol=",
        "genbank": "https://www.ncbi.nlm.nih.gov/nucleotide/",
        "genecards": "https://www.genecards.org/cgi-bin/carddisp.pl?gene=",
        "genedb": "https://www.genedb.org/gene/",
        "genefarm": "https://urgi.versailles.inra.fr/Genefarm/Gene/display_gene.htpl?GENE_ID=",
        "genetree": "http://www.ensembl.org/Multi/GeneTree/Image?db=core;gt=",
        "genewiki": "http://plugins.biogps.org/cgi-bin/wp.cgi?id=",
        "genprop": "https://www.ebi.ac.uk/interpro/genomeproperties/#",
        "geo": "https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=",
        "geonames": "https://www.geonames.org/",
        "ghr": "https://medlineplus.gov/genetics/condition/",
        "giardiadb": "https://giardiadb.org/giardiadb/app/record/gene/",
        "github": "https://github.com/",
        "github.issue": "https://bioregistry.io/resolve/github/issue/",
        "github.pull": "https://bioregistry.io/resolve/github/pull/",
        "gitlab": "https://gitlab.com/",
        "glida.gpcr": "http://pharminfo.pharm.kyoto-u.ac.jp/services/glida/gpcr_information.php?id=",
        "glida.ligand": "http://pharminfo.pharm.kyoto-u.ac.jp/services/glida/ligand_information.php?id=",
        "glycoepitope": "https://www.glycoepitope.jp/epitopes/",
        "glycomedb": "https://glytoucan.org/Structures/Glycans/",
        "glyconavi": "https://glyconavi.org/hub/?id=",
        "glycopost": "https://glycopost.glycosmos.org/entry/",
        "glygen": "https://glygen.org/glycan/",
        "glytoucan": "https://glytoucan.org/Structures/Glycans/",
        "gmd.analyte": "http://gmd.mpimp-golm.mpg.de/Analytes/",
        "gmd.gcms": "http://gmd.mpimp-golm.mpg.de/Spectrums/",
        "gmd.profile": "http://gmd.mpimp-golm.mpg.de/profile/default.aspx?XemlId=",
        "gmd.ref": "http://gmd.mpimp-golm.mpg.de/ReferenceSubstances/",
        "gnpis": "https://urgi.versailles.inra.fr/gnpis/#result/term=",
        "go.model": "http://noctua.geneontology.org/editor/graph/gomodel:",
        "go.resource": "https://bioregistry.io/metaregistry/go/",
        "goa": "https://www.ebi.ac.uk/QuickGO/GProtein?ac=",
        "gold": "https://gold.jgi.doe.gov/resolver?id=",
        "gold.genome": "http://www.genomesonline.org/cgi-bin/GOLD/GOLDCards.cgi?goldstamp=",
        "gold.meta": "http://genomesonline.org/cgi-bin/GOLD/bin/GOLDCards.cgi?goldstamp=",
        "google.patent": "https://www.google.com/patents/",
        "gpcrdb": "https://gpcrdb.org/protein/",
        "gpmdb": "http://gpmdb.thegpm.org/~/dblist_gpmnum/gpmnum=",
        "graingenes.reference": "https://wheat.pw.usda.gov/cgi-bin/GG3/report.cgi?class=reference&name=",
        "graingenes.symbol": "http://wheat.pw.usda.gov/report?class=gene;name=",
        "gramene.gene": "http://www.gramene.org/db/genes/search_gene?acc=",
        "gramene.growthstage": "http://www.gramene.org/db/ontology/search?id=GRO:",
        "gramene.protein": "http://www.gramene.org/db/protein/protein_search?protein_id=",
        "gramene.qtl": "http://www.gramene.org/db/qtl/qtl_display?qtl_accession_id=",
        "gramene.reference": "http://www.gramene.org/db/literature/pub_search?ref_id=",
        "gramene.taxonomy": "http://www.gramene.org/db/ontology/search?id=",
        "grassbase": "https://www.kew.org/data/grasses-db/www/",
        "greengenes": "http://greengenes.lbl.gov/cgi-bin/show_one_record_v2.pl?prokMSA_id=",
        "grid": "https://www.grid.ac/institutes/",
        "grin.taxonomy": "http://www.ars-grin.gov/cgi-bin/npgs/html/taxon.pl?",
        "grsdb": "http://bioinformatics.ramapo.edu/GRSDB2/geneview.php?geneID=",
        "gsfa": "http://www.fao.org/gsfaonline/additives/details.html?id=",
        "gtex": "https://www.gtexportal.org/home/gene/",
        "gtr": "https://www.ncbi.nlm.nih.gov/gtr/conditions/",
        "gudmap": "https://gudmap.org/id/",
        "gwascentral.marker": "https://www.gwascentral.org/marker/",
        "gwascentral.phenotype": "https://www.gwascentral.org/phenotype/",
        "gwascentral.study": "https://www.gwascentral.org/study/",
        "gxa.expt": "https://www.ebi.ac.uk/gxa/experiments/",
        "gxa.gene": "https://www.ebi.ac.uk/gxa/genes/",
        "hamap": "https://hamap.expasy.org/unirule/",
        "hcvdb": "https://euhcvdb.ibcp.fr/euHCVdb/do/displayHCVEntry?primaryAC=",
        "hdr": "http://research.nhgri.nih.gov/apps/homeodomain/web/index.cgi?mode=view&amp;view=proteins&amp;id=",
        "hgmd": "http://www.hgmd.cf.ac.uk/ac/gene.php?gene=",
        "hgnc": "https://www.genenames.org/data/gene-symbol-report/#!/hgnc_id/",
        "hgnc.genefamily": "https://www.genenames.org/cgi-bin/genefamilies/set/",
        "hgnc.genegroup": "https://www.genenames.org/cgi-bin/genefamilies/set/",
        "hgnc.symbol": "https://www.genenames.org/cgi-bin/gene_symbol_report?match=",
        "hinv.locus": "http://h-invitational.jp/hinv/spsoup/locus_view?hix_id=",
        "hinv.protein": "http://h-invitational.jp/hinv/protein/protein_view.cgi?hip_id=",
        "hinv.transcript": "http://h-invitational.jp/hinv/spsoup/transcript_view?hit_id=",
        "hipsci": "http://www.hipsci.org/lines/#/lines/",
        "hmdb": "http://www.hmdb.ca/metabolites/",
        "hms.lincs.compound": "https://lincs.hms.harvard.edu/db/sm/",
        "hogenom": "http://pbil.univ-lyon1.fr/cgi-bin/view-tree.pl?db=HOGENOM5&query=",
        "homd.seq": "http://www.homd.org/modules.php?op=modload&name=GenomeList&file=index&link=detailinfo&seqid=",
        "homd.taxon": "www.homd.org/modules.php?op=modload&name=HOMD&view=dynamic&oraltaxonid=",
        "homologene": "https://www.ncbi.nlm.nih.gov/homologene/",
        "horizon_discovery": "https://horizondiscovery.com/en/search?searchterm=",
        "hpa": "http://www.proteinatlas.org/",
        "hpm.peptide": "http://www.humanproteomemap.org/spectrum.php?pep_id=",
        "hpm.protein": "http://www.humanproteomemap.org/protein.php?hpm_id=",
        "hprd": "http://www.hprd.org/protein/",
        "hpscreg": "https://hpscreg.eu/cell-line/",
        "hsdb": "https://pubchem.ncbi.nlm.nih.gov/source/hsdb/",
        "huge": "https://www.kazusa.or.jp/huge/gfpage/",
        "icd10": "http://apps.who.int/classifications/icd10/browse/2010/en#/",
        "icd10cm": "https://icd.codes/icd10cm/",
        "icd10pcs": "https://www.findacode.com/code.php?set=ICD10PCS&c=",
        "icd11": "https://icd.who.int/browse11/l-m/en#/http://id.who.int/icd/entity/",
        "icd9": "http://www.icd9data.com/getICD9Code.ashx?icd9=",
        "icd9cm": "http://icd9cm.chrisendres.com/index.php?action=search&srchtext=",
        "icdc": "https://caninecommons.cancer.gov/#/study/",
        "icdo": "http://codes.iarc.fr/search.php?cx=009987501641899931167%3A2_7lsevqpdm&cof=FORID%3A9&ie=UTF-8&ie=ISO-8859-1&oe=ISO-8859-1&sa=&q=",
        "iceberg.element": "http://db-mml.sjtu.edu.cn/ICEberg/feature_page.php?ice_id=",
        "iceberg.family": "http://db-mml.sjtu.edu.cn/ICEberg/browse_result.php?type=fam&fam_id=",
        "iclc": "http://www.iclc.it/details/det_list.php?line_id=",
        "ideal": "http://idp1.force.cs.is.nagoya-u.ac.jp/IDEAL/ideal.php?id=",
        "idoo": "http://registry.api.hq.identifiers.org/semanticApi/getRegistryOntology#",
        "idot": "https://biomodels.net/vocab/idot.rdf#",
        "idpo": "https://www.disprot.org/idpo/IDPO:",
        "idr": "https://idr.openmicroscopy.org/search/?query=Name:idr",
        "iedb": "https://www.iedb.org/reference/",
        "igrhcellid": "http://igrcid.ibms.sinica.edu.tw/cgi-bin/cell_line_view.cgi?cl_name=",
        "igsn": "http://igsn.org/",
        "igsr": "https://www.internationalgenome.org/data-portal/sample/",
        "imanis": "https://www.imanislife.com/?s=",
        "imex": "https://www.ebi.ac.uk/intact/imex/main.xhtml?query=",
        "img.gene": "http://img.jgi.doe.gov/cgi-bin/w/main.cgi?section=GeneDetail&gene_oid=",
        "img.taxon": "http://img.jgi.doe.gov/cgi-bin/w/main.cgi?section=TaxonDetail&taxon_oid=",
        "imgt.hla": "https://www.ebi.ac.uk/cgi-bin/imgt/hla/get_allele.cgi?",
        "imgt.ligm": "http://www.imgt.org/ligmdb/view?id=",
        "inaturalist.taxon": "https://www.inaturalist.org/taxa/",
        "inchi": "http://www.chemspider.com/",
        "inchikey": "http://www.chemspider.com/inchikey=",
        "inn": "https://mednet-communities.net/inn/db/ViewINN.aspx?i=",
        "insdc": "https://www.ncbi.nlm.nih.gov/nuccore/",
        "insdc.cds": "http://getentry.ddbj.nig.ac.jp/getentry/dad/",
        "insdc.gca": "https://www.ebi.ac.uk/ena/data/view/",
        "insdc.run": "https://www.ebi.ac.uk/ena/browser/view/",
        "insdc.sra": "https://www.ncbi.nlm.nih.gov/sra/",
        "intact": "https://www.ebi.ac.uk/intact/interaction/",
        "intact.molecule": "https://www.ebi.ac.uk/intact/molecule/",
        "interlex": "https://scicrunch.org/scicrunch/interlex/view/ilx_",
        "interpro": "https://www.ebi.ac.uk/interpro/entry/",
        "ird.segment": "http://www.fludb.org/brc/fluSegmentDetails.do?ncbiGenomicAccession=",
        "irefweb": "http://wodaklab.org/iRefWeb/interaction/show/",
        "iro": "https://legacy.vectorbase.org/ontology-browser?cv=MIRO&t=IRO:",
        "isbn": "http://isbndb.com/search-all.html?kw=",
        "isfinder": "https://www-is.biotoul.fr/scripts/ficheIS.php?name=",
        "isni": "http://www.isni.org/isni/",
        "issn": "https://portal.issn.org/resource/ISSN/",
        "itis": "https://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=",
        "ito": "https://bioportal.bioontology.org/ontologies/ITO/?p=classes&conceptid=https%3A%2F%2Fai-strategies.org%2Fontology%2F",
        "iuphar.family": "http://www.guidetopharmacology.org/GRAC/FamilyDisplayForward?familyId=",
        "iuphar.ligand": "http://www.guidetopharmacology.org/GRAC/LigandDisplayForward?ligandId=",
        "iuphar.receptor": "http://www.guidetopharmacology.org/GRAC/ObjectDisplayForward?objectId=",
        "jax": "https://www.jax.org/strain/",
        "jcggdb": "http://jcggdb.jp/idb/jcggdb/",
        "jcm": "http://www.jcm.riken.go.jp/cgi-bin/jcm/jcm_number?JCM=",
        "jcrb": "https://cellbank.nibiohn.go.jp/~cellbank/en/search_res_det.cgi?RNO=",
        "jcsd": "http://jglobal.jst.go.jp/en/redirect?Nikkaji_No=",
        "jstor": "http://www.jstor.org/stable/",
        "jws": "https://jjj.bio.vu.nl/models/",
        "kaggle": "https://www.kaggle.com/",
        "kclb": "https://cellbank.snu.ac.kr/english/sub/catalog.php?s_cellid=464&page=detail_info&CatNo=59&strQ=",
        "kegg": "http://www.kegg.jp/entry/",
        "kegg.compound": "https://www.kegg.jp/entry/",
        "kegg.disease": "http://www.kegg.jp/entry/",
        "kegg.drug": "https://www.kegg.jp/entry/",
        "kegg.environ": "http://www.kegg.jp/entry/",
        "kegg.enzyme": "http://www.genome.jp/dbget-bin/www_bget?ec:",
        "kegg.genes": "http://www.kegg.jp/entry/",
        "kegg.genome": "http://www.kegg.jp/entry/",
        "kegg.glycan": "https://www.kegg.jp/entry/",
        "kegg.ligand": "http://www.genome.jp/dbget-bin/www_bget?cpd:",
        "kegg.metagenome": "http://www.kegg.jp/entry/",
        "kegg.module": "http://www.kegg.jp/entry/",
        "kegg.orthology": "http://www.kegg.jp/entry/",
        "kegg.pathway": "https://www.kegg.jp/entry/",
        "kegg.reaction": "https://www.kegg.jp/entry/",
        "knapsack": "http://www.knapsackfamily.com/knapsack_core/information.php?word=",
        "lei": "https://www.gleif.org/lei/",
        "lgai.cede": "https://s3.us-east-2.amazonaws.com/lg.cede/",
        "licebase": "https://licebase.org/?q=",
        "ligandbook": "https://ligandbook.org/package/",
        "ligandbox": "http://www.mypresto5.com/ligandbox/cgi-bin/liginf.cgi?id=",
        "ligea": "http://hpc-bioinformatics.cineca.it/fusion/cell_line/",
        "limore": "https://www.picb.ac.cn/limore/cellLines/single?para=",
        "lincs.cell": "http://lincsportal.ccs.miami.edu/cells/#/view/",
        "lincs.data": "http://lincsportal.ccs.miami.edu/datasets/#/view/",
        "lincs.protein": "https://lincs.hms.harvard.edu/db/proteins/",
        "lincs.smallmolecule": "http://lincsportal.ccs.miami.edu/SmallMolecules/#/view/",
        "linguist": "https://raw.githubusercontent.com/github/linguist/master/lib/linguist/languages.yml#",
        "lipidbank": "http://lipidbank.jp/cgi-bin/detail.cgi?id=",
        "lipidmaps": "http://www.lipidmaps.org/data/LMSDRecord.php?LMID=",
        "loinc": "https://loinc.org/",
        "lonza": "https://knowledge.lonza.com/cell?id=",
        "lotus": "https://lotus.naturalproducts.net/compound/lotus_id/",
        "lpt": "http://purl.obolibrary.org/obo/LPT_",
        "lspci": "https://labsyspharm.github.io/lspci/",
        "lter": "https://vocab.lternet.edu/vocab/vocab/index.php?tema=",
        "macie": "https://www.ebi.ac.uk/thornton-srv/databases/cgi-bin/MACiE/entry/getPage.pl?id=",
        "maizegdb.locus": "http://www.maizegdb.org/cgi-bin/displaylocusrecord.cgi?id=",
        "massbank": "http://www.massbank.jp/RecordDisplay?id=",
        "massive": "https://massive.ucsd.edu/ProteoSAFe/QueryMSV?id=",
        "matrixdb": "http://matrixdb.univ-lyon1.fr/cgi-bin/current/newPort?type=biomolecule&value=",
        "mdm": "https://medical-data-models.org/forms/",
        "meddra": "http://purl.bioontology.org/ontology/MEDDRA/",
        "medgen": "https://www.ncbi.nlm.nih.gov/medgen/",
        "merops.clan": "https://www.ebi.ac.uk/merops/cgi-bin/clansum?clan=",
        "merops.entry": "https://www.ebi.ac.uk/merops/cgi-bin/pepsum?id=",
        "merops.family": "https://www.ebi.ac.uk/merops/cgi-bin/famsum?family=",
        "mesh": "https://meshb.nlm.nih.gov/record/ui?ui=",
        "metabolights": "https://www.ebi.ac.uk/metabolights/",
        "metacyc.compound": "https://metacyc.org/compound?orgid=META&id=",
        "metacyc.reaction": "https://metacyc.org/META/NEW-IMAGE?type=REACTION&object=",
        "metanetx.chemical": "https://www.metanetx.org/chem_info/",
        "metanetx.compartment": "https://www.metanetx.org/comp_info/",
        "metanetx.reaction": "https://www.metanetx.org/equa_info/",
        "metlin": "http://metlin.scripps.edu/metabo_info.php?molid=",
        "mex": "https://www.metabolome-express.org/datasetview.php?datasetid=",
        "mge": "http://aclame.ulb.ac.be/perl/Aclame/Genomes/mge_view.cgi?view=info&id=mge:",
        "mgnify.analysis": "https://www.ebi.ac.uk/metagenomics/analyses/",
        "mgnify.proj": "https://www.ebi.ac.uk/metagenomics/projects/",
        "mgnify.samp": "https://www.ebi.ac.uk/metagenomics/samples/",
        "microscope": "http://www.genoscope.cns.fr/agc/microscope/mage/info.php?id=",
        "microsporidia": "http://microsporidiadb.org/micro/showRecord.do?name=GeneRecordClasses.GeneRecordClass&source_id=",
        "millipore": "https://www.merckmillipore.com/catalogue/item/",
        "mimodb": "http://immunet.cn/bdb/index.php/mimoset/",
        "minid": "https://hdl.handle.net/hdl:20.500.12582/",
        "minid.test": "https://hdl.handle.net/hdl:20.500.12633/",
        "mint": "http://mint.bio.uniroma2.it/mint/search/inFrameInteraction.do?interactionAc=",
        "mipmod": "http://bioinfo.iitk.ac.in/MIPModDB/result.php?code=",
        "mirbase": "http://www.mirbase.org/cgi-bin/mirna_entry.pl?acc=",
        "mirbase.family": "http://www.mirbase.org/cgi-bin/mirna_summary.pl?fam=",
        "mirbase.mature": "http://mirbase.org/cgi-bin/mature.pl?mature_acc=",
        "miriam": "https://registry.identifiers.org/registry/",
        "miriam.collection": "https://www.ebi.ac.uk/miriam/main/",
        "miriam.resource": "https://www.ebi.ac.uk/miriam/main/resources/",
        "mirnest": "http://rhesus.amu.edu.pl/mirnest/copy/details.php?id=",
        "mirtarbase": "http://mirtarbase.mbc.nctu.edu.tw/php/detail.php?mirtid=",
        "mlc": "https://www.mlcommons.org/mlc-id/",
        "mmdb": "http://www.ncbi.nlm.nih.gov/Structure/mmdb/mmdbsrv.cgi?uid=",
        "mmmp.biomaps": "http://www.mmmp.org/MMMP/public/biomap/viewBiomap.mmmp?id=",
        "mmp.cat": "https://mmp.sfb.uit.no/databases/marcat/#/records/",
        "mmp.db": "https://mmp.sfb.uit.no/databases/mardb/#/records/",
        "mmp.fun": "https://mmp.sfb.uit.no/databases/marfun/#/records/",
        "mmp.ref": "https://mmp.sfb.uit.no/databases/marref/#/records/",
        "mmrrc": "http://www.mmrrc.org/catalog/getSDS.php?mmrrc_id=",
        "mobidb": "https://mobidb.org/",
        "modeldb": "http://senselab.med.yale.edu/ModelDB/ShowModel.asp?model=",
        "molbase": "http://www.molbase.com/en/index.php?app=search&search_keyword=",
        "molmedb": "https://molmedb.upol.cz/mol/",
        "morpheus": "https://morpheus.gitlab.io/models/",
        "mpid": "http://www.jcvi.org/mpidb/experiment.php?interaction_id=",
        "msigdb": "https://biopragmatics.github.io/providers/msigdb/",
        "multicellds": "http://multicellds.org/MultiCellDB/",
        "multicellds.cell_line": "http://multicellds.org/MultiCellDB/",
        "multicellds.collection": "http://multicellds.org/MultiCellDB/",
        "multicellds.snapshot": "http://multicellds.org/MultiCellDB/",
        "mw.project": "http://www.metabolomicsworkbench.org/data/DRCCMetadata.php?Mode=Project&ProjectID=",
        "mw.study": "http://www.metabolomicsworkbench.org/data/DRCCMetadata.php?Mode=Study&StudyID=",
        "myco.lepra": "http://mycobrowser.epfl.ch/leprosysearch.php?gene+name=",
        "myco.marinum": "http://mycobrowser.epfl.ch/marinosearch.php?gene+name=",
        "myco.smeg": "http://mycobrowser.epfl.ch/smegmasearch.php?gene+name=",
        "myco.tuber": "http://tuberculist.epfl.ch/quicksearch.php?gene+name=",
        "mycobank": "http://www.mycobank.org/Biolomics.aspx?Table=Mycobank&MycoBankNr_=",
        "mzspec": "http://proteomecentral.proteomexchange.org/usi/?usi=mzspec:",
        "napdi": "https://repo.napdi.org/study/",
        "napp": "http://rna.igmors.u-psud.fr/NAPP/Niveau2.php?specie=",
        "narcis": "http://www.narcis.nl/publication/RecordID/",
        "nasc": "http://arabidopsis.info/StockInfo?NASC_id=",
        "nbrc": "http://www.nbrc.nite.go.jp/NBRC2/NBRCCatalogueDetailServlet?ID=NBRC&CAT=",
        "ncats.bioplanet": "https://tripod.nih.gov/bioplanet/detail.jsp?pid=bioplanet_",
        "ncats.drug": "https://drugs.ncats.io/drug/",
        "ncbi.assembly": "https://www.ncbi.nlm.nih.gov/assembly/",
        "ncbi.genome": "https://www.ncbi.nlm.nih.gov/genome/",
        "ncbi.resource": "https://bioregistry.io/metaregistry/ncbi/",
        "ncbibook": "https://www.ncbi.nlm.nih.gov/books/",
        "ncim": "http://ncim.nci.nih.gov/ncimbrowser/ConceptReport.jsp?dictionary=NCI%20MetaThesaurus&code=",
        "ndc": "http://www.hipaaspace.com/Medical_Billing/Coding/National.Drug.Codes/",
        "ndex": "https://www.ndexbio.org/viewer/networks/",
        "nemo": "https://assets.nemoarchive.org/",
        "neurolex": "http://uri.neuinfo.org/nif/nifstd/nlx_",
        "neuromorpho": "http://neuromorpho.org/neuron_info.jsp?neuron_name=",
        "neuronames": "http://braininfo.rprc.washington.edu/centraldirectory.aspx?ID=",
        "neurondb": "http://senselab.med.yale.edu/NeuronDB/NeuronProp.aspx?id=",
        "neurovault.collection": "https://neurovault.org/collections/",
        "neurovault.image": "https://neurovault.org/images/",
        "nextdb": "http://nematode.lab.nig.ac.jp/db2/ShowCloneInfo.php?clone=",
        "nextprot": "https://www.nextprot.org/db/entry/",
        "nextprot.family": "https://www.nextprot.org/term/FA-",
        "ngl": "https://genelab-data.ndc.nasa.gov/genelab/accession/",
        "nhcdr": "https://stemcells.nindsgenetics.org?line=",
        "niaest": "http://lgsun.grc.nia.nih.gov/cgi-bin/pro3?sname1=",
        "nif.ext": "http://uri.neuinfo.org/nif/nifstd/nifext_",
        "nif.std": "http://uri.neuinfo.org/nif/nifstd/",
        "nihreporter.project": "https://reporter.nih.gov/project-details/",
        "nlm": "https://www.ncbi.nlm.nih.gov/nlmcatalog/",
        "nlx.anat": "http://uri.neuinfo.org/nif/nifstd/nlx_anat_",
        "nlx.br": "http://uri.neuinfo.org/nif/nifstd/nlx_br_",
        "nlx.cell": "http://uri.neuinfo.org/nif/nifstd/nlx_cell_",
        "nlx.chem": "http://uri.neuinfo.org/nif/nifstd/nlx_chem_",
        "nlx.dys": "http://uri.neuinfo.org/nif/nifstd/nlx_dys_",
        "nlx.func": "http://uri.neuinfo.org/nif/nifstd/nlx_func_",
        "nlx.inv": "http://uri.neuinfo.org/nif/nifstd/nlx_inv_",
        "nlx.mol": "http://uri.neuinfo.org/nif/nifstd/nlx_mol_",
        "nlx.oen": "http://uri.neuinfo.org/nif/nifstd/oen_",
        "nlx.org": "http://uri.neuinfo.org/nif/nifstd/nlx_organ_",
        "nlx.qual": "http://uri.neuinfo.org/nif/nifstd/nlx_qual_",
        "nlx.res": "http://uri.neuinfo.org/nif/nifstd/nlx_res_",
        "nlx.sub": "http://uri.neuinfo.org/nif/nifstd/nlx_subcell_",
        "nmdc": "https://drs.microbiomedata.org/objects/",
        "nmrshiftdb2": "https://nmrshiftdb.nmr.uni-koeln.de/molecule/",
        "noaa": "https://www.fisheries.noaa.gov/species/",
        "noncodev3": "http://www.noncode.org/NONCODERv3/ncrna.php?ncid=",
        "noncodev4.gene": "http://www.bioinfo.org/NONCODEv4/show_gene.php?id=",
        "noncodev4.rna": "http://www.bioinfo.org/NONCODEv4/show_rna.php?id=",
        "norine": "http://bioinfo.lifl.fr/norine/result.jsp?ID=",
        "novus": "https://www.novusbio.com/products/",
        "nuclearbd": "http://www.receptors.org/nucleardb/proteins/",
        "nucleotide": "https://www.ncbi.nlm.nih.gov/nuccore/",
        "oa": "http://www.w3.org/ns/oa#",
        "obo": "http://purl.obolibrary.org/obo/",
        "oboInOwl": "http://www.geneontology.org/formats/oboInOwl#",
        "occ": "https://w3id.org/oc/corpus/",
        "oci": "https://w3id.org/oc/oci/",
        "ocid": "https://ocid.ontochem.com/prefname?ocid=",
        "oclc": "https://www.worldcat.org/oclc/",
        "odc.sci": "https://odc-sci.org/data/",
        "odc.tbi": "https://odc-tbi.org/data/",
        "om": "http://www.ontology-of-units-of-measure.org/resource/om-2/",
        "oma.grp": "https://omabrowser.org/cgi-bin/gateway.pl?f=DisplayGroup&p1=",
        "oma.hog": "https://omabrowser.org/oma/hog/HOG:",
        "oma.protein": "https://omabrowser.org/cgi-bin/gateway.pl?f=DisplayEntry&p1=",
        "omim": "https://omim.org/entry/",
        "omim.ps": "https://omim.org/phenotypicSeries/",
        "oncotree": "http://oncotree.mskcc.org/api/tumorTypes/search/code/",
        "opb": "http://purl.bioontology.org/ontology/OPB?conceptid=http%3A%2F%2Fbhi.washington.edu%2FOPB%23OPB_",
        "openalex": "https://openalex.org/",
        "opm": "http://opm.phar.umich.edu/protein.php?pdbid=",
        "orcid": "https://orcid.org/",
        "ordb": "http://senselab.med.yale.edu/ORDB/Data/",
        "oridb.sacch": "http://cerevisiae.oridb.org/details.php?id=",
        "oridb.schizo": "http://pombe.oridb.org/details.php?id=",
        "orphanet": "http://www.orpha.net/consor/cgi-bin/OC_Exp.php?Lng=EN&Expert=",
        "orphanet.ordo": "http://www.orpha.net/ORDO/Orphanet_",
        "orthodb": "http://cegg.unige.ch/orthodb/results?searchtext=",
        "oryzabase.gene": "http://www.shigen.nig.ac.jp/rice/oryzabaseV4/gene/detail/",
        "oryzabase.mutant": "http://www.shigen.nig.ac.jp/rice/oryzabaseV4/strain/inducedMutationLine/detail/",
        "oryzabase.reference": "https://shigen.nig.ac.jp/rice/oryzabase/reference/detail/",
        "oryzabase.stage": "http://www.shigen.nig.ac.jp/rice/oryzabaseV4/devstageineachorgan/detail/",
        "oryzabase.strain": "http://www.shigen.nig.ac.jp/rice/oryzabaseV4/strain/wildCore/detail/",
        "otl": "http://oryzatagline.cirad.fr/cgi-bin/general_mutant.pl?line=",
        "otol": "https://tree.opentreeoflife.org/taxonomy/browse?id=",
        "owl": "http://www.w3.org/2002/07/owl#",
        "paleodb": "http://fossilworks.org/?a=taxonInfo&taxon_no=",
        "panther.family": "http://www.pantherdb.org/panther/family.do?clsAccession=",
        "panther.node": "http://www.pantree.org/node/annotationNode.jsp?id=",
        "panther.pathway": "http://www.pantherdb.org/pathway/pathwayDiagram.jsp?catAccession=",
        "panther.pthcmp": "http://www.pantherdb.org/pathway/pathCatDetail.do?clsAccession=",
        "pass2": "http://caps.ncbs.res.in/cgi-bin/pass2//show_sf.py?sf_id=",
        "pathbank": "https://pathbank.org/view/",
        "pathwaycommons": "http://www.pathwaycommons.org/pc/record2.do?id=",
        "pav": "http://purl.org/pav/",
        "paxdb.organism": "http://pax-db.org/#!species/",
        "paxdb.protein": "http://pax-db.org/#!protein/",
        "pazar": "http://www.pazar.info/cgi-bin/tf_search.cgi?geneID=",
        "pdb": "https://www.wwpdb.org/pdb?id=",
        "pdb-ccd": "https://www.ebi.ac.uk/pdbe-srv/pdbechem/chemicalCompound/show/",
        "pdb.ligand": "http://www.rcsb.org/pdb/ligand/ligandsummary.do?hetId=",
        "pdc.study": "https://pdc.cancer.gov/pdc/study/",
        "ped": "https://proteinensemble.org/",
        "ped.ensemble": "https://proteinensemble.org/",
        "peptideatlas": "https://db.systemsbiology.net/sbeams/cgi/PeptideAtlas/Summarize_Peptide?query=QUERY&searchForThis=",
        "peptideatlas.dataset": "http://www.peptideatlas.org/PASS/",
        "perkinelmer": "https://www.perkinelmer.com/searchresult?searchName=",
        "peroxibase": "http://peroxibase.toulouse.inra.fr/browse/process/view_perox.php?id=",
        "pfam": "https://pfam.xfam.org/family/",
        "pfam.clan": "https://pfam.xfam.org/clan/",
        "pfr": "http://repository.topdownproteomics.org/proteoforms/",
        "pgs": "https://www.pgscatalog.org/pgs/",
        "pgx": "https://progenetix.org/services/ids/",
        "pharmacodb.cell": "https://pharmacodb.ca/cell_lines/",
        "pharmacodb.dataset": "https://pharmacodb.ca/datasets/",
        "pharmacodb.tissue": "https://pharmacodb.ca/tissues/",
        "pharmgkb.disease": "http://www.pharmgkb.org/disease/",
        "pharmgkb.drug": "http://www.pharmgkb.org/drug/",
        "pharmgkb.gene": "http://www.pharmgkb.org/gene/",
        "pharmgkb.pathways": "http://www.pharmgkb.org/pathway/",
        "phenolexplorer": "http://phenol-explorer.eu/foods/",
        "phenx": "https://www.phenxtoolkit.org/protocols/view/",
        "phosphosite.protein": "http://www.phosphosite.org/proteinAction.do?id=",
        "phosphosite.residue": "http://www.phosphosite.org/siteAction.do?id=",
        "phylomedb": "http://phylomedb.org/?seqid=",
        "phytozome.locus": "http://www.phytozome.net/genePage.php?crown&method=0&search=1&detail=1&searchText=locusname:",
        "pid.pathway": "http://pid.nci.nih.gov/search/pathway_landing.shtml?what=graphic&jpg=on&pathway_id=",
        "pigqtldb": "https://www.animalgenome.org/QTLdb/q?id=QTL_ID:",
        "pina": "http://cbg.garvan.unsw.edu.au/pina/interactome.oneP.do?showExtend=null&ac=",
        "piroplasma": "http://piroplasmadb.org/piro/showRecord.do?name=GeneRecordClasses.GeneRecordClass&source_id=",
        "pirsf": "https://pir.georgetown.edu/cgi-bin/ipcSF?id=",
        "pkdb": "https://pk-db.com/data/",
        "planttfdb": "http://planttfdb.cbi.pku.edu.cn/tf.php?uid=",
        "plasmodb": "http://plasmodb.org/plasmo/showRecord.do?name=GeneRecordClasses.GeneRecordClass&source_id=",
        "pmap.cutdb": "http://cutdb.burnham.org/relation/show/",
        "pmc": "http://europepmc.org/articles/",
        "pmdb": "https://bioinformatics.cineca.it/PMDB/user//search.php?idsearch=",
        "pmp": "http://www.proteinmodelportal.org/query/uniprot/",
        "pmr": "https://models.physiomeproject.org/exposure/",
        "pombase": "http://www.pombase.org/spombe/result/",
        "ppr": "https://europepmc.org/article/ppr/",
        "prefixcommons": "https://bioregistry.io/metaregistry/prefixcommons/",
        "pride": "https://www.ebi.ac.uk/pride/archive/assays/",
        "pride.project": "https://www.ebi.ac.uk/pride/archive/projects/",
        "probonto": "https://www.ebi.ac.uk/ols/ontologies/probonto/terms?short_form=",
        "prodom": "http://prodom.prabi.fr/prodom/current/cgi-bin/request.pl?question=DBEN&query=",
        "proglyc": "http://www.proglycprot.org/detail.aspx?ProId=",
        "prosite": "https://prosite.expasy.org/",
        "protclustdb": "https://www.ncbi.nlm.nih.gov/sites/entrez?Db=proteinclusters&Cmd=DetailsSearch&Term=",
        "proteomicsdb.peptide": "https://www.proteomicsdb.org/#human/proteinDetails/",
        "protonet.cluster": "http://www.protonet.cs.huji.ac.il/requested/cluster_card.php?cluster=",
        "protonet.proteincard": "http://www.protonet.cs.huji.ac.il/requested/protein_card.php?protein_id=",
        "prov": "http://www.w3.org/ns/prov#",
        "pseudomonas": "http://www.pseudomonas.com/feature/show/?locus_tag=",
        "psipar": "https://www.ebi.ac.uk/ontology-lookup/?termId=",
        "pubchem.bioassay": "https://pubchem.ncbi.nlm.nih.gov/bioassay/",
        "pubchem.cell": "https://pubchem.ncbi.nlm.nih.gov/cell/",
        "pubchem.classification": "https://pubchem.ncbi.nlm.nih.gov/classification/#hid=",
        "pubchem.compound": "https://pubchem.ncbi.nlm.nih.gov/compound/",
        "pubchem.element": "https://pubchem.ncbi.nlm.nih.gov/element/",
        "pubchem.substance": "https://pubchem.ncbi.nlm.nih.gov/substance/",
        "publons.researcher": "https://publons.com/researcher/",
        "pubmed": "https://www.ncbi.nlm.nih.gov/pubmed/",
        "px": "http://proteomecentral.proteomexchange.org/cgi/GetDataset?ID=",
        "pypi": "https://pypi.org/project/",
        "qtldb": "https://www.animalgenome.org/QTLdb/q?id=QTL_ID:",
        "qudt": "http://qudt.org/schema/qudt#",
        "rapdb.locus": "https://rapdb.dna.affrc.go.jp/viewer/gbrowse_details/irgsp1?name=",
        "rapdb.transcript": "https://rapdb.dna.affrc.go.jp/viewer/gene_detail/irgsp1?name=",
        "rbk": "https://www.rebuildingakidney.org/id/",
        "rcb": "https://cellbank.brc.riken.jp/cell_bank/CellInfo/?cellNo=",
        "rdf": "http://www.w3.org/1999/02/22-rdf-syntax-ns#",
        "rdfs": "http://www.w3.org/2000/01/rdf-schema#",
        "re3data": "https://www.re3data.org/repository/",
        "reactome": "https://reactome.org/content/detail/",
        "receptome.family": "http://www.receptome.org/families/Frameset_family.asp?ProtType=Receptor&FamId=",
        "refseq": "https://www.ncbi.nlm.nih.gov/protein/",
        "repeatsdb.protein": "https://repeatsdb.org/protein/",
        "repeatsdb.structure": "https://repeatsdb.org/structure/",
        "rfam": "https://rfam.xfam.org/family/",
        "rgd": "http://rgd.mcw.edu/rgdweb/report/gene/main.html?id=",
        "rgd.qtl": "http://rgd.mcw.edu/rgdweb/report/qtl/main.html?id=",
        "rgd.strain": "http://rgd.mcw.edu/rgdweb/report/strain/main.html?id=",
        "rhea": "https://www.rhea-db.org/rhea/",
        "ricegap": "http://rice.plantbiology.msu.edu/cgi-bin/ORF_infopage.cgi?&orf=",
        "ricenetdb.compound": "http://bis.zju.edu.cn/ricenetdb/compounddetails.php?ID=",
        "ricenetdb.gene": "http://bis.zju.edu.cn/ricenetdb/genedetails.php?ID=",
        "ricenetdb.mirna": "http://bis.zju.edu.cn/ricenetdb/miRNAdetails.php?ID=",
        "ricenetdb.protein": "http://bis.zju.edu.cn/ricenetdb/proteindetails.php?ID=",
        "ricenetdb.reaction": "http://bis.zju.edu.cn/ricenetdb/reactiondetails.php?ID=",
        "rnacentral": "https://rnacentral.org/rna/",
        "rnamod": "http://rna.rega.kuleuven.be/cgi-bin/rnamods/rnashow.pl?",
        "rnamods": "http://mods.rna.albany.edu/mods/modifications/view/",
        "rouge": "https://www.kazusa.or.jp/rouge/gfpage/",
        "rrid": "https://scicrunch.org/resolver/RRID:",
        "runbiosimulations": "https://run.biosimulations.org/simulations/",
        "rxnorm": "https://mor.nlm.nih.gov/RxNav/search?searchBy=RXCUI&searchTerm=",
        "sabiork.compound": "http://sabiork.h-its.org/newSearch?q=",
        "sabiork.ec": "http://sabiork.h-its.org/newSearch?q=ecnumber:",
        "sabiork.kineticrecord": "http://sabiork.h-its.org/kineticLawEntry.jsp?viewData=true&kinlawid=",
        "sabiork.reaction": "http://sabiork.h-its.org/reacdetails.jsp?reactid=",
        "salk": "https://abrc.osu.edu/stocks/number/SALK_",
        "sasbdb": "http://www.sasbdb.org/data/",
        "schem": "https://biopragmatics.github.io/providers/schem/",
        "schema": "https://schema.org/",
        "scholia.resource": "https://bioregistry.io/metaregistry/scholia/",
        "scomp": "https://biopragmatics.github.io/providers/scomp/",
        "scop": "http://scop.mrc-lmb.cam.ac.uk/scop/search.cgi?sunid=",
        "scr": "http://scicrunch.org/resolver/SCR_",
        "scretf": "http://stormo.wustl.edu/ScerTF/details/",
        "sdbs": "http://riodb01.ibase.aist.go.jp/sdbs/cgi-bin/cre_frame_disp.cgi?sdbsno=",
        "sdis": "https://biopragmatics.github.io/providers/sdis/",
        "sedml.format": "https://sed-ml.org/urns.html#format:",
        "sedml.language": "https://sed-ml.org/urns.html#language:",
        "seed": "http://seed-viewer.theseed.org/seedviewer.cgi?page=Subsystems&subsystem=",
        "seed.compound": "http://modelseed.org/biochem/compounds/",
        "seed.reaction": "http://modelseed.org/biochem/reactions/",
        "sfam": "https://biopragmatics.github.io/providers/sfam/",
        "sgd": "https://www.yeastgenome.org/locus/",
        "sgd.pathways": "http://pathway.yeastgenome.org/YEAST/new-image?type=PATHWAY&object=",
        "sgn": "http://solgenomics.net/phenome/locus_display.pl?locus_id=",
        "sheepqtldb": "https://www.animalgenome.org/QTLdb/q?id=QTL_ID:",
        "sider.drug": "http://sideeffects.embl.de/drugs/",
        "sider.effect": "http://sideeffects.embl.de/se/",
        "sigmaaldrich": "https://www.sigmaaldrich.com/US/en/product/sigma/",
        "signaling-gateway": "http://www.signaling-gateway.org/molecule/query?afcsid=",
        "signor": "https://signor.uniroma2.it/relation_result.php?id=",
        "sio": "https://www.ebi.ac.uk/ols/ontologies/sio/terms?short_form=",
        "sisu": "http://search.sisuproject.fi/#/variant/",
        "sitex": "http://www-bionet.sscc.ru/sitex/index.php?siteid=",
        "skip": "https://skip.stemcellinformatics.org/SKIPSearch/cell_line_detail?accession=",
        "skos": "http://www.w3.org/2004/02/skos/core#",
        "slkbase": "https://sumlineknowledgebase.com/?page_id=",
        "smart": "http://smart.embl-heidelberg.de/smart/do_annotation.pl?DOMAIN=",
        "smid": "https://smid-db.org/smid/",
        "smpdb": "https://smpdb.ca/view/",
        "snp2tfbs": "https://ccg.epfl.ch/cgi-bin/snp2tfbs/snpviewer_form_parser.cgi?snpid=",
        "soybase": "http://www.soybase.org/sbt/search/search_results.php?category=SNP&search_term=",
        "spdx": "https://spdx.org/licenses/",
        "splash": "http://mona.fiehnlab.ucdavis.edu/#/spectra/splash/",
        "spp": "https://www.signalingpathways.org/datasets/dataset.jsf?doi=",
        "sprint": "http://130.88.97.239/cgi-bin/dbbrowser/sprint/searchprintss.cgi?display_opts=Prints&category=None&queryform=false&regexpr=off&prints_accn=",
        "ssbd.dataset": "https://ssbd.riken.jp/database/dataset/",
        "ssbd.project": "https://ssbd.riken.jp/database/project/",
        "stap": "http://psb.kobic.re.kr/STAP/refinement1/result.php?search=",
        "stitch": "http://stitch.embl.de/interactions/",
        "storedb": "https://www.storedb.org/?",
        "storedb.dataset": "https://www.storedb.org/?STOREDB:DATASET",
        "storedb.file": "https://www.storedb.org/store_v3/download.jsp?fileId=",
        "storedb.study": "https://www.storedb.org/store_v3/study.jsp?studyId=",
        "string": "http://string.embl.de/interactions/",
        "sty": "http://purl.bioontology.org/ontology/STY/",
        "subtilist": "http://genolist.pasteur.fr/SubtiList/genome.cgi?external_query+",
        "subtiwiki": "http://www.subtiwiki.uni-goettingen.de/wiki/index.php/",
        "sugarbind": "http://sugarbind.expasy.org/",
        "supfam": "http://supfam.org/SUPERFAMILY/cgi-bin/scop.cgi?ipid=",
        "swh": "https://archive.softwareheritage.org/browse/swh:",
        "swiss-model": "https://swissmodel.expasy.org/repository/uniprot/",
        "swisslipid": "https://www.swisslipids.org/#/entity/SLM:",
        "swissregulon": "http://swissregulon.unibas.ch/query/",
        "t3db": "http://www.t3db.org/toxins/",
        "tair.gene": "http://arabidopsis.org/servlets/TairObject?accession=Gene:",
        "tair.locus": "http://www.arabidopsis.org/servlets/TairObject?accession=Locus:",
        "tair.protein": "http://arabidopsis.org/servlets/TairObject?accession=AASequence:",
        "tarbase": "http://diana.imis.athena-innovation.gr/DianaTools/index.php?r=tarbase/index&mirnas=",
        "tcb": "https://www.pirbright.ac.uk/node/",
        "tcdb": "http://www.tcdb.org/search/result.php?tc=",
        "tfclass": "http://tfclass.bioinf.med.uni-goettingen.de/?tfclass=",
        "tgd": "http://ciliate.org/index.php/feature/details/",
        "tgn": "http://vocab.getty.edu/page/tgn/",
        "thermofisher": "https://www.thermofisher.com/antibody/product/",
        "tigrfam": "https://www.ncbi.nlm.nih.gov/cdd?term=",
        "tngb": "http://biobanknetwork.telethon.it/Sample/View?sampleId=",
        "tol": "http://tolweb.org/",
        "topdb": "http://topdb.enzim.hu/?m=show&id=",
        "topfind": "http://clipserve.clip.ubc.ca/topfind/proteins/",
        "toxoplasma": "http://toxodb.org/toxo/showRecord.do?name=GeneRecordClasses.GeneRecordClass&source_id=",
        "transyt": "https://transyt.bio.di.uminho.pt/reactions/",
        "treefam": "http://www.treefam.org/family/",
        "trichdb": "http://trichdb.org/trichdb/showRecord.do?name=GeneRecordClasses.GeneRecordClass&source_id=",
        "tritrypdb": "http://tritrypdb.org/tritrypdb/showRecord.do?name=GeneRecordClasses.GeneRecordClass&source_id=",
        "ttd.drug": "http://bidd.nus.edu.sg/group/TTD/ZFTTDDRUG.asp?ID=",
        "ttd.target": "http://bidd.nus.edu.sg/group/TTD/ZFTTDDetail.asp?ID=",
        "ubio.namebank": "http://www.ubio.org/browser/details.php?namebankID=",
        "ucsc": "ftp://hgdownload.cse.ucsc.edu/goldenPath/",
        "umbbd.compound": "http://eawag-bbd.ethz.ch/servlets/pageservlet?ptype=c&compID=",
        "umbbd.enzyme": "http://eawag-bbd.ethz.ch/servlets/pageservlet?ptype=ep&enzymeID=",
        "umbbd.reaction": "http://eawag-bbd.ethz.ch/servlets/pageservlet?ptype=r&reacID=",
        "umbbd.rule": "http://www.umbbd.ethz.ch/servlets/rule.jsp?rule=",
        "unigene": "http://www.ncbi.nlm.nih.gov/UniGene/clust.cgi?UGID=",
        "unii": "http://fdasis.nlm.nih.gov/srs/srsdirect.jsp?regno=",
        "unimod": "http://www.unimod.org/modifications_view.php?editid1=",
        "uniparc": "https://www.ebi.ac.uk/cgi-bin/dbfetch?db=uniparc&id=",
        "unipathway.compound": "http://www.grenoble.prabi.fr/obiwarehouse/unipathway/upc?upid=",
        "unipathway.reaction": "http://www.grenoble.prabi.fr/obiwarehouse/unipathway/ucr?upid=",
        "uniprot": "http://purl.uniprot.org/uniprot/",
        "uniprot.chain": "http://purl.uniprot.org/annotation/",
        "uniprot.disease": "https://www.uniprot.org/diseases/",
        "uniprot.isoform": "http://www.uniprot.org/uniprot/",
        "uniprot.journal": "https://www.uniprot.org/journals/",
        "uniprot.keyword": "https://www.uniprot.org/keywords/",
        "uniprot.location": "https://www.uniprot.org/locations/",
        "uniprot.ptm": "https://biopragmatics.github.io/providers/uniprot.ptm/",
        "uniprot.resource": "https://www.uniprot.org/database/",
        "uniprot.tissue": "https://www.uniprot.org/tissues/",
        "uniref": "https://www.uniprot.org/uniref/",
        "unirule": "https://www.uniprot.org/unirule/",
        "unists": "https://www.ncbi.nlm.nih.gov/genome/sts/sts.cgi?uid=",
        "unite": "http://unite.ut.ee/bl_forw.php?nimi=",
        "uspto": "http://patft.uspto.gov/netacgi/nph-Parser?Sect2=PTO1&Sect2=HITOFF&p=1&u=/netahtml/PTO/search-bool.html&r=1&f=G&l=50&d=PALL&RefSrch=yes&Query=PN/",
        "validatordb": "https://webchem.ncbr.muni.cz/Platform/ValidatorDb/ByStructure/",
        "vbase2": "http://www.vbase2.org/vgene.php?id=",
        "vbrc": "http://vbrc.org/gene_detail.asp?gene_id=",
        "vcell": "https://vcell.org/biomodel-",
        "vectorbase": "https://vectorbase.org/vectorbase/app/record/gene/",
        "vegbank": "http://vegbank.org/cite/",
        "vfb": "http://virtualflybrain.org/reports/",
        "vfdb.gene": "http://www.mgc.ac.cn/cgi-bin/VFs/gene.cgi?GeneID=",
        "vfdb.genus": "http://www.mgc.ac.cn/cgi-bin/VFs/genus.cgi?Genus=",
        "vgnc": "https://vertebrate.genenames.org/data/gene-symbol-report/#!/vgnc_id/",
        "viaf": "http://viaf.org/viaf/",
        "virsirna": "http://crdd.osdd.net/servers/virsirnadb/record.php?details=",
        "vmhgene": "https://www.vmh.life/#gene/",
        "vmhmetabolite": "https://www.vmh.life/#metabolite/",
        "vmhreaction": "https://www.vmh.life/#reaction/",
        "void": "http://rdfs.org/ns/void#",
        "vso": "https://bioportal.bioontology.org/ontologies/VSO/?p=classes&conceptid=http%3A%2F%2Fpurl.obolibrary.org%2Fobo%2FVSO_",
        "vz": "https://viralzone.expasy.org/",
        "wb.rnai": "https://www.wormbase.org/species/c_elegans/rnai/",
        "webelements": "https://www.webelements.com/",
        "wgs84": "http://www.w3.org/2003/01/geo/wgs84_pos#",
        "wikidata": "http://www.wikidata.org/entity/",
        "wikidata.property": "https://www.wikidata.org/wiki/Property:",
        "wikipathways": "http://www.wikipathways.org/instance/",
        "wikipedia.en": "http://en.wikipedia.org/wiki/",
        "worfdb": "http://worfdb.dfci.harvard.edu/index.php?search_type=name&page=showresultrc&race_query=",
        "wormpep": "https://www.wormbase.org/db/seq/protein?name=",
        "worms": "http://www.marinespecies.org/aphia.php?p=taxdetails&id=",
        "wwf.ecoregion": "https://www.worldwildlife.org/ecoregions/",
        "xenbase": "https://www.xenbase.org/entry/",
        "ximbio": "https://ximbio.com/reagent/",
        "xl": "http://purl.obolibrary.org/obo/XL_",
        "xmetdb": "http://www.xmetdb.org/xmetdb/protocol/",
        "xml": "http://www.w3.org/XML/1998/namespace#",
        "xsd": "http://www.w3.org/2001/XMLSchema#",
        "ydpm": "http://www-deletion.stanford.edu/cgi-bin/YDPM/YDPM_search.cgi?thelist=",
        "yeastintron": "http://intron.ucsc.edu/cgi-bin/yeast4.3/intronreports.pl?outputformat=full&amp;orfName=",
        "yetfasco": "http://yetfasco.ccbr.utoronto.ca/showPFM.php?mot=",
        "yid": "http://metarray.ucsc.edu/cgi-bin/intron/yirIntrondb?orfName=",
        "ymdb": "http://www.ymdb.ca/compounds/",
        "yrcpdr": "http://yeastrc.org/pdr/viewProtein.do?id=",
        "zenodo.record": "https://zenodo.org/record/",
        "zfin": "http://zfin.org/",
        "zinc": "http://zinc15.docking.org/substances/"
    }
}"""



"""SSSOM parsers."""

import json
import logging
import re
import typing
from collections import Counter
from pathlib import Path
from typing import Any, Callable, Dict, List, Optional, TextIO, Tuple, Union, cast
from urllib.request import urlopen
from xml.dom import Node, minidom
from xml.dom.minidom import Document

import numpy as np
import pandas as pd
import validators
import yaml
from deprecation import deprecated
from linkml_runtime.loaders.json_loader import JSONLoader
from rdflib import Graph, URIRef

# from .sssom_datamodel import Mapping, MappingSet
from sssom_schema import Mapping, MappingSet

from sssom.constants import (
    CONFIDENCE,
    CURIE_MAP,
    DEFAULT_MAPPING_PROPERTIES,
    LICENSE,
    MAPPING_JUSTIFICATION,
    MAPPING_JUSTIFICATION_UNSPECIFIED,
    MAPPING_SET_ID,
    OBJECT_ID,
    OBJECT_LABEL,
    OBJECT_SOURCE,
    OBJECT_SOURCE_ID,
    OWL_EQUIV_CLASS,
    PREDICATE_ID,
    RDFS_SUBCLASS_OF,
    SUBJECT_ID,
    SUBJECT_LABEL,
    SUBJECT_SOURCE,
    SUBJECT_SOURCE_ID,
    SSSOMSchemaView,
)

from .context import (
    DEFAULT_LICENSE,
    DEFAULT_MAPPING_SET_ID,
    add_built_in_prefixes_to_prefix_map,
    get_default_metadata,
)
from .sssom_document import MappingSetDocument
from .typehints import Metadata, MetadataType, PrefixMap
from .util import (
    PREFIX_MAP_KEY,
    SSSOM_DEFAULT_RDF_SERIALISATION,
    URI_SSSOM_MAPPINGS,
    MappingSetDataFrame,
    NoCURIEException,
    curie_from_uri,
    get_file_extension,
    is_multivalued_slot,
    raise_for_bad_path,
    read_pandas,
    to_mapping_set_dataframe,
)

# * DEPRECATED methods *****************************************


@deprecated(
    deprecated_in="0.3.10",
    removed_in="0.3.11",
    details="Use 'parse_sssom_table' instead.",
)
def read_sssom_table(
    file_path: Union[str, Path],
    prefix_map: Optional[PrefixMap] = None,
    meta: Optional[MetadataType] = None,
    **kwargs,
) -> MappingSetDataFrame:
    """DEPRECATE."""
    return parse_sssom_table(
        file_path=file_path, prefix_map=prefix_map, meta=meta, kwargs=kwargs
    )


@deprecated(
    deprecated_in="0.3.10",
    removed_in="0.3.11",
    details="Use 'parse_sssom_rdf' instead.",
)
def read_sssom_rdf(
    file_path: str,
    prefix_map: Dict[str, str] = None,
    meta: Dict[str, str] = None,
    serialisation=SSSOM_DEFAULT_RDF_SERIALISATION,
    **kwargs,
) -> MappingSetDataFrame:
    """DEPRECATE."""
    return parse_sssom_rdf(
        file_path=file_path,
        prefix_map=prefix_map,
        meta=meta,
        serialisation=serialisation,
        kwargs=kwargs,
    )


@deprecated(
    deprecated_in="0.3.10",
    removed_in="0.3.11",
    details="Use 'parse_sssom_json' instead.",
)
def read_sssom_json(
    file_path: str,
    prefix_map: Dict[str, str] = None,
    meta: Dict[str, str] = None,
    **kwargs
    # mapping_predicates: Optional[List[str]] = None,
) -> MappingSetDataFrame:
    """DEPRECATE."""
    return parse_sssom_json(
        file_path=file_path, prefix_map=prefix_map, meta=meta, kwarg=kwargs
    )


# * *******************************************************
# Parsers (from file)


def parse_sssom_table(
    file_path: Union[str, Path],
    prefix_map: Optional[PrefixMap] = None,
    meta: Optional[MetadataType] = None,
    **kwargs
    # mapping_predicates: Optional[List[str]] = None,
) -> MappingSetDataFrame:
    """Parse a TSV to a :class:`MappingSetDocument` to a :class:`MappingSetDataFrame`."""
    raise_for_bad_path(file_path)
    df = read_pandas(file_path)
    # if mapping_predicates:
    #     # Filter rows based on presence of predicate_id list provided.
    #     df = df[df["predicate_id"].isin(mapping_predicates)]

    # If SSSOM external metadata is provided, merge it with the internal metadata
    sssom_metadata = _read_metadata_from_table(file_path)

    if sssom_metadata:
        if meta:
            for k, v in meta.items():
                if k in sssom_metadata:
                    if sssom_metadata[k] != v:
                        logging.warning(
                            f"SSSOM internal metadata {k} ({sssom_metadata[k]}) "
                            f"conflicts with provided ({meta[k]})."
                        )
                else:
                    logging.info(
                        f"Externally provided metadata {k}:{v} is added to metadata set."
                    )
                    sssom_metadata[k] = v
        meta = sssom_metadata

        if "curie_map" in sssom_metadata:
            if prefix_map:
                for k, v in prefix_map.items():
                    if k in sssom_metadata[CURIE_MAP]:
                        if sssom_metadata[CURIE_MAP][k] != v:
                            logging.warning(
                                f"SSSOM prefix map {k} ({sssom_metadata[CURIE_MAP][k]}) "
                                f"conflicts with provided ({prefix_map[k]})."
                            )
                    else:
                        logging.info(
                            f"Externally provided metadata {k}:{v} is added to metadata set."
                        )
                        sssom_metadata[CURIE_MAP][k] = v
            prefix_map = sssom_metadata[CURIE_MAP]

    meta_all = _get_prefix_map_and_metadata(prefix_map=prefix_map, meta=meta)
    msdf = from_sssom_dataframe(
        df, prefix_map=meta_all.prefix_map, meta=meta_all.metadata
    )
    return msdf


def parse_sssom_rdf(
    file_path: str,
    prefix_map: Dict[str, str] = None,
    meta: Dict[str, str] = None,
    serialisation=SSSOM_DEFAULT_RDF_SERIALISATION,
    **kwargs
    # mapping_predicates: Optional[List[str]] = None,
) -> MappingSetDataFrame:
    """Parse a TSV to a :class:`MappingSetDocument` to a :class:`MappingSetDataFrame`."""
    raise_for_bad_path(file_path)
    metadata = _get_prefix_map_and_metadata(prefix_map=prefix_map, meta=meta)

    g = Graph()
    g.parse(file_path, format=serialisation)
    msdf = from_sssom_rdf(g, prefix_map=metadata.prefix_map, meta=metadata.metadata)
    # df: pd.DataFrame = msdf.df
    # if mapping_predicates and not df.empty():
    #     msdf.df = df[df["predicate_id"].isin(mapping_predicates)]
    return msdf


def parse_sssom_json(
    file_path: str,
    prefix_map: Dict[str, str] = None,
    meta: Dict[str, str] = None,
    **kwargs
    # mapping_predicates: Optional[List[str]] = None,
) -> MappingSetDataFrame:
    """Parse a TSV to a :class:`MappingSetDocument` to a  :class`MappingSetDataFrame`."""
    raise_for_bad_path(file_path)
    metadata = _get_prefix_map_and_metadata(prefix_map=prefix_map, meta=meta)

    with open(file_path) as json_file:
        jsondoc = json.load(json_file)
    msdf = from_sssom_json(
        jsondoc=jsondoc, prefix_map=metadata.prefix_map, meta=metadata.metadata
    )
    # df: pd.DataFrame = msdf.df
    # if mapping_predicates and not df.empty():
    #     msdf.df = df[df["predicate_id"].isin(mapping_predicates)]
    return msdf


# Import methods from external file formats


def parse_obographs_json(
    file_path: str,
    prefix_map: Dict[str, str] = None,
    meta: Dict[str, str] = None,
    mapping_predicates: Optional[List[str]] = None,
) -> MappingSetDataFrame:
    """Parse an obographs file as a JSON object and translates it into a MappingSetDataFrame.

    :param file_path: The path to the obographs file
    :param prefix_map: an optional prefix map
    :param meta: an optional dictionary of metadata elements
    :param mapping_predicates: an optional list of mapping predicates that should be extracted
    :return: A SSSOM MappingSetDataFrame
    """
    raise_for_bad_path(file_path)

    _xmetadata = _get_prefix_map_and_metadata(prefix_map=prefix_map, meta=meta)

    with open(file_path) as json_file:
        jsondoc = json.load(json_file)

    return from_obographs(
        jsondoc,
        prefix_map=_xmetadata.prefix_map,
        meta=_xmetadata.metadata,
        mapping_predicates=mapping_predicates,
    )


def _get_prefix_map_and_metadata(
    prefix_map: Optional[PrefixMap] = None, meta: Optional[MetadataType] = None
) -> Metadata:
    default_metadata = get_default_metadata()

    if prefix_map is None:
        logging.warning(
            "No prefix map provided (not recommended), trying to use defaults.."
        )
        prefix_map = default_metadata.prefix_map

    if meta is None:
        meta = default_metadata.metadata
    else:
        if prefix_map and PREFIX_MAP_KEY in meta:
            logging.info(
                "Prefix map provided as parameter, but SSSOM file provides its own prefix map. "
                "Prefix map provided externally is disregarded in favour of the prefix map in the SSSOM file."
            )
            prefix_map = cast(PrefixMap, meta[PREFIX_MAP_KEY])

    return Metadata(prefix_map=prefix_map, metadata=meta)


def _address_multivalued_slot(k: str, v: Any) -> Union[str, List[str]]:
    if is_multivalued_slot(k) and v is not None and isinstance(v, str):
        # IF k is multivalued, then v = List[values]
        return [s.strip() for s in v.split("|")]
    else:
        return v


def _init_mapping_set(meta: Optional[MetadataType]) -> MappingSet:
    license = DEFAULT_LICENSE
    mapping_set_id = DEFAULT_MAPPING_SET_ID
    if meta is not None:
        if MAPPING_SET_ID in meta.keys():
            mapping_set_id = meta[MAPPING_SET_ID]
        if LICENSE in meta.keys():
            license = meta[LICENSE]
    return MappingSet(mapping_set_id=mapping_set_id, license=license)


def _get_mdict_ms_and_bad_attrs(
    row: pd.Series, ms: MappingSet, bad_attrs: Counter
) -> Tuple[dict, MappingSet, Counter]:

    mdict = {}
    sssom_schema_object = (
        SSSOMSchemaView.instance if SSSOMSchemaView.instance else SSSOMSchemaView()
    )
    for k, v in row.items():
        if v and v == v:
            ok = False
            if k:
                k = str(k)
            v = _address_multivalued_slot(k, v)
            # if hasattr(Mapping, k):
            if k in sssom_schema_object.mapping_slots:
                mdict[k] = v
                ok = True
            # if hasattr(MappingSet, k):
            if k in sssom_schema_object.mapping_set_slots:
                ms[k] = v
                ok = True
            if not ok:
                bad_attrs[k] += 1
    return (mdict, ms, bad_attrs)


def parse_alignment_xml(
    file_path: str,
    prefix_map: Dict[str, str],
    meta: Dict[str, str],
    mapping_predicates: Optional[List[str]] = None,
) -> MappingSetDataFrame:
    """Parse a TSV -> MappingSetDocument -> MappingSetDataFrame."""
    raise_for_bad_path(file_path)

    metadata = _get_prefix_map_and_metadata(prefix_map=prefix_map, meta=meta)
    logging.info("Loading from alignment API")
    xmldoc = minidom.parse(file_path)
    msdf = from_alignment_minidom(
        xmldoc,
        prefix_map=metadata.prefix_map,
        meta=metadata.metadata,
        mapping_predicates=mapping_predicates,
    )
    return msdf


# Readers (from object)


def from_sssom_dataframe(
    df: pd.DataFrame,
    prefix_map: Optional[PrefixMap] = None,
    meta: Optional[MetadataType] = None,
) -> MappingSetDataFrame:
    """Convert a dataframe to a MappingSetDataFrame.

    :param df: A mappings dataframe
    :param prefix_map: A prefix map
    :param meta: A metadata dictionary
    :return: MappingSetDataFrame
    """
    prefix_map = _ensure_prefix_map(prefix_map)

    # Need to revisit this solution.
    # This is to address: A value is trying to be set on a copy of a slice from a DataFrame
    if CONFIDENCE in df.columns:
        df2 = df.copy()
        df2[CONFIDENCE].replace(r"^\s*$", np.NaN, regex=True, inplace=True)
        df = df2

    mlist: List[Mapping] = []
    ms = _init_mapping_set(meta)
    bad_attrs: typing.Counter[str] = Counter()
    for _, row in df.iterrows():
        mdict, ms, bad_attrs = _get_mdict_ms_and_bad_attrs(row, ms, bad_attrs)
        mlist.append(_prepare_mapping(Mapping(**mdict)))
    for k, v in bad_attrs.most_common():
        logging.warning(f"No attr for {k} [{v} instances]")
    # the autogenerated code's type annotations are _really_ messy. This is in fact okay,
    # so with a heavy heart we employ type:ignore
    ms.mappings = mlist  # type:ignore
    _set_metadata_in_mapping_set(mapping_set=ms, metadata=meta)
    doc = MappingSetDocument(mapping_set=ms, prefix_map=prefix_map)
    return to_mapping_set_dataframe(doc)


def from_sssom_rdf(
    g: Graph,
    prefix_map: Optional[PrefixMap] = None,
    meta: Optional[MetadataType] = None,
) -> MappingSetDataFrame:
    """Convert an SSSOM RDF graph into a SSSOM data table.

    :param g: the Graph (rdflib)
    :param prefix_map: A dictionary containing the prefix map, defaults to None
    :param meta: Potentially additional metadata, defaults to None
    :return: MappingSetDataFrame object
    """
    prefix_map = _ensure_prefix_map(prefix_map)

    ms = _init_mapping_set(meta)
    mlist: List[Mapping] = []
    for sx, px, ox in g.triples((None, URIRef(URI_SSSOM_MAPPINGS), None)):
        mdict: Dict[str, Any] = {}
        # TODO replace with g.predicate_objects()
        for _s, p, o in g.triples((ox, None, None)):
            if isinstance(p, URIRef):
                try:
                    p_id = curie_from_uri(p, prefix_map)
                    k = None

                    if p_id.startswith("sssom:"):
                        k = p_id.replace("sssom:", "")
                    elif p_id == "owl:annotatedProperty":
                        k = "predicate_id"
                    elif p_id == "owl:annotatedTarget":
                        k = "object_id"
                    elif p_id == "owl:annotatedSource":
                        k = "subject_id"

                    if isinstance(o, URIRef):
                        v: Any
                        v = curie_from_uri(o, prefix_map)
                    else:
                        v = o.toPython()
                    if k:
                        v = _address_multivalued_slot(k, v)
                        mdict[k] = v

                except NoCURIEException as e:
                    logging.warning(e)
        if mdict:
            m = _prepare_mapping(Mapping(**mdict))
            if _is_valid_mapping(m):
                mlist.append(m)
            else:
                logging.warning(
                    f"While trying to prepare a mapping for {mdict}, something went wrong. "
                    f"One of subject_id, object_id or predicate_id was missing."
                )
        else:
            logging.warning(
                f"While trying to prepare a mapping for {sx},{px}, {ox}, something went wrong. "
                f"This usually happens when a critical prefix_map entry is missing."
            )

    ms.mappings = mlist  # type: ignore
    _set_metadata_in_mapping_set(mapping_set=ms, metadata=meta)
    mdoc = MappingSetDocument(mapping_set=ms, prefix_map=prefix_map)
    return to_mapping_set_dataframe(mdoc)


def from_sssom_json(
    jsondoc: Union[str, dict, TextIO],
    prefix_map: Dict[str, str],
    meta: Dict[str, str] = None,
) -> MappingSetDataFrame:
    """Load a mapping set dataframe from a JSON object.

    :param jsondoc: JSON document
    :param prefix_map: Prefix map
    :param meta: metadata
    :return: MappingSetDataFrame object
    """
    prefix_map = _ensure_prefix_map(prefix_map)
    mapping_set = cast(
        MappingSet, JSONLoader().load(source=jsondoc, target_class=MappingSet)
    )

    _set_metadata_in_mapping_set(mapping_set, metadata=meta)
    mapping_set_document = MappingSetDocument(
        mapping_set=mapping_set, prefix_map=prefix_map
    )
    return to_mapping_set_dataframe(mapping_set_document)


def from_alignment_minidom(
    dom: Document,
    prefix_map: PrefixMap,
    meta: MetadataType,
    mapping_predicates: Optional[List[str]] = None,
) -> MappingSetDataFrame:
    """Read a minidom Document object.

    :param dom: XML (minidom) object
    :param prefix_map: A prefix map
    :param meta: Optional meta data
    :param mapping_predicates: Optional list of mapping predicates to extract
    :return: MappingSetDocument
    :raises ValueError: for alignment format: xml element said, but not set to yes. Only XML is supported!
    """
    # FIXME: should be prefix_map =  _check_prefix_map(prefix_map)
    _ensure_prefix_map(prefix_map)
    ms = _init_mapping_set(meta)
    mlist: List[Mapping] = []
    # bad_attrs = {}

    if not mapping_predicates:
        mapping_predicates = DEFAULT_MAPPING_PROPERTIES

    alignments = dom.getElementsByTagName("Alignment")
    for n in alignments:
        for e in n.childNodes:
            if e.nodeType == Node.ELEMENT_NODE:
                node_name = e.nodeName
                if node_name == "map":
                    cell = e.getElementsByTagName("Cell")
                    for c_node in cell:
                        mdict = _cell_element_values(
                            c_node, prefix_map, mapping_predicates=mapping_predicates
                        )
                        if mdict:
                            m = _prepare_mapping(mdict)
                            mlist.append(m)
                        else:
                            logging.warning(
                                f"While trying to prepare a mapping for {c_node}, something went wrong. "
                                f"This usually happens when a critical prefix_map entry is missing."
                            )

                elif node_name == "xml":
                    if e.firstChild.nodeValue != "yes":
                        raise ValueError(
                            "Alignment format: xml element said, but not set to yes. Only XML is supported!"
                        )
                elif node_name == "onto1":
                    ms[SUBJECT_SOURCE_ID] = e.firstChild.nodeValue
                elif node_name == "onto2":
                    ms[OBJECT_SOURCE_ID] = e.firstChild.nodeValue
                elif node_name == "uri1":
                    ms[SUBJECT_SOURCE] = e.firstChild.nodeValue
                elif node_name == "uri2":
                    ms[OBJECT_SOURCE] = e.firstChild.nodeValue

    ms.mappings = mlist  # type: ignore
    _set_metadata_in_mapping_set(mapping_set=ms, metadata=meta)
    mapping_set_document = MappingSetDocument(mapping_set=ms, prefix_map=prefix_map)
    return to_mapping_set_dataframe(mapping_set_document)


def _get_obographs_predicate_id(obographs_predicate: str):
    if obographs_predicate == "is_a":
        return RDFS_SUBCLASS_OF
    return obographs_predicate


def from_obographs(
    jsondoc: Dict,
    prefix_map: PrefixMap,
    meta: Optional[MetadataType] = None,
    mapping_predicates: Optional[List[str]] = None,
) -> MappingSetDataFrame:
    """Convert a obographs json object to an SSSOM data frame.

    :param jsondoc: The JSON object representing the ontology in obographs format
    :param prefix_map: The prefix map to be used
    :param meta: Any additional metadata that needs to be added to the resulting SSSOM data frame, defaults to None
    :param mapping_predicates: Optional list of mapping predicates to extract
    :raises Exception: When there is no CURIE
    :return: An SSSOM data frame (MappingSetDataFrame)
    """
    _ensure_prefix_map(prefix_map)
    ms = _init_mapping_set(meta)
    mlist: List[Mapping] = []
    # bad_attrs = {}

    if not mapping_predicates:
        mapping_predicates = DEFAULT_MAPPING_PROPERTIES

    labels = {}

    # Build a dictionary of labels to populate _label columns
    if "graphs" in jsondoc:
        for g in jsondoc["graphs"]:
            if "nodes" in g:
                for n in g["nodes"]:
                    nid = n["id"]
                    if "lbl" in n:
                        label = n["lbl"]
                    else:
                        label = ""
                    labels[nid] = label

    if "graphs" in jsondoc:
        for g in jsondoc["graphs"]:
            if "nodes" in g:
                for n in g["nodes"]:
                    nid = n["id"]
                    if "lbl" in n:
                        label = n["lbl"]
                    else:
                        label = ""
                    if "meta" in n:
                        if (
                            "xrefs" in n["meta"]
                            and "http://www.geneontology.org/formats/oboInOwl#hasDbXref"
                            in mapping_predicates
                        ):
                            for xref in n["meta"]["xrefs"]:
                                xref_id = xref["val"]
                                mdict: Dict[str, Any] = {}
                                try:
                                    mdict[SUBJECT_ID] = curie_from_uri(nid, prefix_map)
                                    mdict[OBJECT_ID] = curie_from_uri(
                                        xref_id, prefix_map
                                    )
                                    mdict[SUBJECT_LABEL] = label
                                    mdict[PREDICATE_ID] = "oboInOwl:hasDbXref"
                                    mdict[
                                        MAPPING_JUSTIFICATION
                                    ] = MAPPING_JUSTIFICATION_UNSPECIFIED
                                    mlist.append(Mapping(**mdict))
                                except NoCURIEException as e:
                                    # FIXME this will cause all sorts of ragged Mappings
                                    logging.warning(e)
                        if "basicPropertyValues" in n["meta"]:
                            for value in n["meta"]["basicPropertyValues"]:
                                pred = value["pred"]
                                if pred in mapping_predicates:
                                    xref_id = value["val"]
                                    mdict = {}
                                    try:
                                        mdict[SUBJECT_ID] = curie_from_uri(
                                            nid, prefix_map
                                        )
                                        mdict[OBJECT_ID] = curie_from_uri(
                                            xref_id, prefix_map
                                        )
                                        mdict[SUBJECT_LABEL] = label
                                        mdict[PREDICATE_ID] = curie_from_uri(
                                            pred, prefix_map
                                        )
                                        mdict[
                                            MAPPING_JUSTIFICATION
                                        ] = MAPPING_JUSTIFICATION_UNSPECIFIED
                                        mlist.append(Mapping(**mdict))
                                    except NoCURIEException as e:
                                        # FIXME this will cause ragged mappings
                                        logging.warning(e)
            if "edges" in g:
                for edge in g["edges"]:
                    mdict = {}
                    subject_id = edge["sub"]
                    predicate_id = _get_obographs_predicate_id(edge["pred"])
                    object_id = edge["obj"]
                    if predicate_id in mapping_predicates:
                        mdict[SUBJECT_ID] = curie_from_uri(subject_id, prefix_map)
                        mdict[OBJECT_ID] = curie_from_uri(object_id, prefix_map)
                        mdict[SUBJECT_LABEL] = (
                            labels[subject_id] if subject_id in labels.keys() else ""
                        )
                        mdict[OBJECT_LABEL] = (
                            labels[object_id] if object_id in labels.keys() else ""
                        )
                        mdict[PREDICATE_ID] = curie_from_uri(predicate_id, prefix_map)
                        mdict[MAPPING_JUSTIFICATION] = MAPPING_JUSTIFICATION_UNSPECIFIED
                        mlist.append(Mapping(**mdict))
            if "equivalentNodesSets" in g and OWL_EQUIV_CLASS in mapping_predicates:
                for equivalents in g["equivalentNodesSets"]:
                    if "nodeIds" in equivalents:
                        for ec1 in equivalents["nodeIds"]:
                            for ec2 in equivalents["nodeIds"]:
                                if ec1 != ec2:
                                    mdict = {}
                                    mdict[SUBJECT_ID] = curie_from_uri(ec1, prefix_map)
                                    mdict[OBJECT_ID] = curie_from_uri(ec2, prefix_map)
                                    mdict[PREDICATE_ID] = curie_from_uri(
                                        OWL_EQUIV_CLASS, prefix_map
                                    )
                                    mdict[
                                        MAPPING_JUSTIFICATION
                                    ] = MAPPING_JUSTIFICATION_UNSPECIFIED
                                    mdict[SUBJECT_LABEL] = (
                                        labels[ec1] if ec1 in labels.keys() else ""
                                    )
                                    mdict[OBJECT_LABEL] = (
                                        labels[ec2] if ec2 in labels.keys() else ""
                                    )
                                    mlist.append(Mapping(**mdict))
    else:
        raise Exception("No graphs element in obographs file, wrong format?")

    ms.mappings = mlist  # type: ignore
    _set_metadata_in_mapping_set(mapping_set=ms, metadata=meta)
    mdoc = MappingSetDocument(mapping_set=ms, prefix_map=prefix_map)
    return to_mapping_set_dataframe(mdoc)


# All from_* take as an input a python object (data frame, json, etc) and return a MappingSetDataFrame
# All read_* take as an input a a file handle and return a MappingSetDataFrame (usually wrapping a from_* method)


def get_parsing_function(input_format: Optional[str], filename: str) -> Callable:
    """Return appropriate parser function based on input format of file.

    :param input_format: File format
    :param filename: Filename
    :raises Exception: Unknown file format
    :return: Appropriate 'read' function
    """
    if input_format is None:
        input_format = get_file_extension(filename)
    if input_format == "tsv":
        return parse_sssom_table
    elif input_format == "rdf":
        return parse_sssom_rdf
    elif input_format == "json":
        return parse_sssom_json
    elif input_format == "alignment-api-xml":
        return parse_alignment_xml
    elif input_format == "obographs-json":
        return parse_obographs_json
    else:
        raise Exception(f"Unknown input format: {input_format}")


def _ensure_prefix_map(prefix_map: Optional[PrefixMap] = None) -> PrefixMap:
    if not prefix_map:
        raise Exception("No valid prefix_map provided")
    else:
        return add_built_in_prefixes_to_prefix_map(prefix_map)


def _prepare_mapping(mapping: Mapping) -> Mapping:
    p = mapping.predicate_id
    if p == "sssom:superClassOf":
        mapping.predicate_id = "rdfs:subClassOf"
        return _swap_object_subject(mapping)
    return mapping


def _swap_object_subject(mapping: Mapping) -> Mapping:
    members = [
        attr.replace("subject_", "")
        for attr in dir(mapping)
        if not callable(getattr(mapping, attr))
        and not attr.startswith("__")
        and attr.startswith("subject_")
    ]
    for var in members:
        subject_val = getattr(mapping, "subject_" + var)
        object_val = getattr(mapping, "object_" + var)
        setattr(mapping, "subject_" + var, object_val)
        setattr(mapping, "object_" + var, subject_val)
    return mapping


def _read_metadata_from_table(path: Union[str, Path]) -> Dict[str, Any]:
    if isinstance(path, Path) or not validators.url(path):
        with open(path) as file:
            yamlstr = ""
            for line in file:
                if line.startswith("#"):
                    yamlstr += re.sub("^#", "", line)
                else:
                    break
    else:
        response = urlopen(path)
        yamlstr = ""
        for lin in response:
            line = lin.decode("utf-8")
            if line.startswith("#"):
                yamlstr += re.sub("^#", "", line)
            else:
                break

    if yamlstr:
        meta = yaml.safe_load(yamlstr)
        logging.info(f"Meta={meta}")
        return meta
    return {}


def _is_valid_mapping(m: Mapping) -> bool:
    return bool(m.predicate_id and m.object_id and m.subject_id)


def _set_metadata_in_mapping_set(
    mapping_set: MappingSet, metadata: Optional[MetadataType] = None
) -> None:
    if metadata is None:
        logging.info("Tried setting metadata but none provided.")
    else:
        for k, v in metadata.items():
            if k != PREFIX_MAP_KEY:
                mapping_set[k] = v


def _cell_element_values(
    cell_node, prefix_map: PrefixMap, mapping_predicates
) -> Optional[Mapping]:
    mdict: Dict[str, Any] = {}
    for child in cell_node.childNodes:
        if child.nodeType == Node.ELEMENT_NODE:
            try:
                if child.nodeName == "entity1":
                    mdict[SUBJECT_ID] = curie_from_uri(
                        child.getAttribute("rdf:resource"), prefix_map
                    )
                elif child.nodeName == "entity2":
                    mdict[OBJECT_ID] = curie_from_uri(
                        child.getAttribute("rdf:resource"), prefix_map
                    )
                elif child.nodeName == "measure":
                    mdict[CONFIDENCE] = child.firstChild.nodeValue
                elif child.nodeName == "relation":
                    relation = child.firstChild.nodeValue
                    if (relation == "=") and (OWL_EQUIV_CLASS in mapping_predicates):
                        mdict[PREDICATE_ID] = "owl:equivalentClass"
                    else:
                        logging.warning(f"{relation} not a recognised relation type.")
                else:
                    logging.warning(
                        f"Unsupported alignment api element: {child.nodeName}"
                    )
            except NoCURIEException as e:
                logging.warning(e)

    mdict[MAPPING_JUSTIFICATION] = MAPPING_JUSTIFICATION_UNSPECIFIED

    m = Mapping(**mdict)
    if _is_valid_mapping(m):
        return m
    else:
        return None


# The following methods dont really belong in the parser package..


def to_mapping_set_document(msdf: MappingSetDataFrame) -> MappingSetDocument:
    """Convert a MappingSetDataFrame to a MappingSetDocument."""
    if not msdf.prefix_map:
        raise Exception("No valid prefix_map provided")

    mlist: List[Mapping] = []
    ms = _init_mapping_set(msdf.metadata)
    bad_attrs: Counter = Counter()
    if msdf.df is not None:
        for _, row in msdf.df.iterrows():
            mdict, ms, bad_attrs = _get_mdict_ms_and_bad_attrs(row, ms, bad_attrs)
            m = _prepare_mapping(Mapping(**mdict))
            mlist.append(m)
    for k, v in bad_attrs.items():
        logging.warning(f"No attr for {k} [{v} instances]")
    ms.mappings = mlist  # type: ignore
    if msdf.metadata is not None:
        for k, v in msdf.metadata.items():
            if k != PREFIX_MAP_KEY:
                ms[k] = _address_multivalued_slot(k, v)
    return MappingSetDocument(mapping_set=ms, prefix_map=msdf.prefix_map)


def split_dataframe(
    msdf: MappingSetDataFrame,
) -> Dict[str, MappingSetDataFrame]:
    """Group the mapping set dataframe into several subdataframes by prefix.

    :param msdf: MappingSetDataFrame object
    :raises RuntimeError: DataFrame object within MappingSetDataFrame is None
    :return: Mapping object
    """
    if msdf.df is None:
        raise RuntimeError
    subject_prefixes = set(msdf.df[SUBJECT_ID].str.split(":", n=1, expand=True)[0])
    object_prefixes = set(msdf.df[OBJECT_ID].str.split(":", n=1, expand=True)[0])
    relations = set(msdf.df[PREDICATE_ID])
    return split_dataframe_by_prefix(
        msdf=msdf,
        subject_prefixes=subject_prefixes,
        object_prefixes=object_prefixes,
        relations=relations,
    )


def split_dataframe_by_prefix(
    msdf: MappingSetDataFrame, subject_prefixes, object_prefixes, relations
) -> Dict[str, MappingSetDataFrame]:
    """Split a mapping set dataframe by prefix.

    :param msdf: An SSSOM MappingSetDataFrame
    :param subject_prefixes: a list of prefixes pertaining to the subject
    :param object_prefixes: a list of prefixes pertaining to the object
    :param relations: a list of relations of interest
    :return: a dict of SSSOM data frame names to MappingSetDataFrame
    """
    df = msdf.df
    prefix_map = msdf.prefix_map
    meta = msdf.metadata
    splitted = {}
    for pre_subj in subject_prefixes:
        for pre_obj in object_prefixes:
            for rel in relations:
                relpre = rel.split(":")[0]
                relppost = rel.split(":")[1]
                split_name = f"{pre_subj.lower()}_{relppost.lower()}_{pre_obj.lower()}"
                if df is not None:
                    dfs = df[
                        (df[SUBJECT_ID].str.startswith(pre_subj + ":"))
                        & (df[PREDICATE_ID] == rel)
                        & (df[OBJECT_ID].str.startswith(pre_obj + ":"))
                    ]
                if pre_subj in prefix_map and pre_obj in prefix_map and len(dfs) > 0:
                    cm = {
                        pre_subj: prefix_map[pre_subj],
                        pre_obj: prefix_map[pre_obj],
                        relpre: prefix_map[relpre],
                    }
                    msdf = from_sssom_dataframe(dfs, prefix_map=cm, meta=meta)
                    splitted[split_name] = msdf
                else:
                    logging.warning(
                        f"Not adding {split_name} because there is a missing prefix ({pre_subj}, {pre_obj}), "
                        f"or no matches ({len(dfs)} matches found)"
                    )
    return splitted
